@@ -15,16 +15,16 @@
  *
  */
 
-#include "gz/rendering/ogre/OgreCamera.hh"
-#include "gz/rendering/ogre/OgreConversions.hh"
-#include "gz/rendering/ogre/OgreIncludes.hh"
-#include "gz/rendering/ogre/OgreMaterial.hh"
-#include "gz/rendering/ogre/OgreRenderTarget.hh"
-#include "gz/rendering/ogre/OgreScene.hh"
-#include "gz/rendering/ogre/OgreSelectionBuffer.hh"
-#include "gz/rendering/Utils.hh"
+#include "ignition/rendering/ogre/OgreCamera.hh"
+#include "ignition/rendering/ogre/OgreConversions.hh"
+#include "ignition/rendering/ogre/OgreIncludes.hh"
+#include "ignition/rendering/ogre/OgreMaterial.hh"
+#include "ignition/rendering/ogre/OgreRenderTarget.hh"
+#include "ignition/rendering/ogre/OgreScene.hh"
+#include "ignition/rendering/ogre/OgreSelectionBuffer.hh"
+#include "ignition/rendering/Utils.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -38,6 +38,12 @@ OgreCamera::~OgreCamera()
   this->Destroy();
 }
 
+//////////////////////////////////////////////////
+Ogre::Camera *OgreCamera::Camera() const
+{
+  return this->ogreCamera;
+}
+
 //////////////////////////////////////////////////
 void OgreCamera::Destroy()
 {
@@ -201,7 +207,7 @@ void OgreCamera::SetSelectionBuffer()
 }
 
 //////////////////////////////////////////////////
-VisualPtr OgreCamera::VisualAt(const math::Vector2i
+VisualPtr OgreCamera::VisualAt(const ignition::math::Vector2i
     &_mousePos)
 {
   VisualPtr result;
@@ -217,7 +223,7 @@ VisualPtr OgreCamera::VisualAt(const math::Vector2i
   }
 
   float ratio = screenScalingFactor();
-  math::Vector2i mousePos(
+  ignition::math::Vector2i mousePos(
       static_cast<int>(std::rint(ratio * _mousePos.X())),
       static_cast<int>(std::rint(ratio * _mousePos.Y())));
 
@@ -305,3 +311,10 @@ double OgreCamera::FarClip() const
   else
     return 0;
 }
+
+//////////////////////////////////////////////////
+void OgreCamera::SetVisibilityMask(uint32_t _mask)
+{
+  BaseCamera::SetVisibilityMask(_mask);
+  this->renderTexture->SetVisibilityMask(_mask);
+}
