@@ -15,21 +15,21 @@
  *
 */
 
-#include <gz/common/Mesh.hh>
-#include <gz/common/MeshManager.hh>
-#include <gz/common/SubMesh.hh>
+#include <ignition/common/Mesh.hh>
+#include <ignition/common/MeshManager.hh>
+#include <ignition/common/SubMesh.hh>
 
-#include <gz/math/Color.hh>
-#include <gz/math/Helpers.hh>
-#include <gz/math/Vector3.hh>
+#include <ignition/math/Color.hh>
+#include <ignition/math/Helpers.hh>
+#include <ignition/math/Vector3.hh>
 
-#include "gz/rendering/RenderTypes.hh"
-#include "gz/rendering/ogre/OgreCamera.hh"
-#include "gz/rendering/ogre/OgreGpuRays.hh"
+#include "ignition/rendering/RenderTypes.hh"
+#include "ignition/rendering/ogre/OgreCamera.hh"
+#include "ignition/rendering/ogre/OgreGpuRays.hh"
 
 /// \internal
 /// \brief Private data for the OgreGpuRays class
-class gz::rendering::OgreGpuRaysPrivate
+class ignition::rendering::OgreGpuRaysPrivate
 {
   /// \brief Event triggered when new gpu rays range data are available.
   /// \param[in] _frame New frame containing raw gpu rays data.
@@ -37,7 +37,7 @@ class gz::rendering::OgreGpuRaysPrivate
   /// \param[in] _height Height of frame.
   /// \param[in] _channels Number of channels
   /// \param[in] _format Format of frame.
-  public: gz::common::EventT<void(const float *,
+  public: ignition::common::EventT<void(const float *,
                unsigned int, unsigned int, unsigned int,
                const std::string &)> newGpuRaysFrame;
 
@@ -50,7 +50,7 @@ class gz::rendering::OgreGpuRaysPrivate
   /// \brief Pointer to Ogre material for the first rendering pass.
   public: Ogre::Material *matFirstPass = nullptr;
 
-  /// \brief Pointer to Ogre material for the sencod rendering pass.
+  /// \brief Pointer to Ogre material for the second rendering pass.
   public: Ogre::Material *matSecondPass = nullptr;
 
   /// \brief Temporary pointer to the current material.
@@ -126,7 +126,7 @@ class gz::rendering::OgreGpuRaysPrivate
   public: unsigned int cameraCount = 1;
 };
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -710,7 +710,7 @@ void OgreGpuRays::CreateOrthoCam()
   }
 
   Ogre::SceneNode *rootSceneNode = std::dynamic_pointer_cast<
-      OgreNode>(this->scene->RootVisual())->Node();
+      ignition::rendering::OgreNode>(this->scene->RootVisual())->Node();
   this->dataPtr->pitchNodeOrtho = rootSceneNode->createChildSceneNode();
   this->dataPtr->pitchNodeOrtho->attachObject(this->dataPtr->orthoCam);
 
@@ -893,7 +893,7 @@ void OgreGpuRays::CreateCanvas()
       this->Name() + "second_pass_canvas");
 
   Ogre::SceneNode *visualSceneNode =  std::dynamic_pointer_cast<
-    OgreNode>(this->dataPtr->visual)->Node();
+    ignition::rendering::OgreNode>(this->dataPtr->visual)->Node();
 
   Ogre::Node *visualParent = visualSceneNode->getParent();
   if (visualParent != nullptr)
@@ -902,7 +902,7 @@ void OgreGpuRays::CreateCanvas()
   }
   this->dataPtr->pitchNodeOrtho->addChild(visualSceneNode);
 
-  // Convert mesh from common::Mesh to mesh and add it to
+  // Convert mesh from common::Mesh to rendering::mesh and add it to
   // the canvas visual
   MeshPtr renderingMesh = this->scene->CreateMesh(
       this->dataPtr->undistMesh);
@@ -913,7 +913,7 @@ void OgreGpuRays::CreateCanvas()
 
   MaterialPtr canvasMaterial =
     this->scene->CreateMaterial(this->Name() + "_green");
-  canvasMaterial->SetAmbient(math::Color(0, 1, 0, 1));
+  canvasMaterial->SetAmbient(ignition::math::Color(0, 1, 0, 1));
   this->dataPtr->visual->SetMaterial(canvasMaterial);
 
   this->dataPtr->visual->SetVisible(true);
@@ -975,7 +975,7 @@ void OgreGpuRays::notifyRenderSingleObject(Ogre::Renderable *_rend,
 }
 
 //////////////////////////////////////////////////
-common::ConnectionPtr OgreGpuRays::ConnectNewGpuRaysFrame(
+ignition::common::ConnectionPtr OgreGpuRays::ConnectNewGpuRaysFrame(
     std::function<void(const float *_frame, unsigned int _width,
     unsigned int _height, unsigned int _channels,
     const std::string &/*_format*/)> _subscriber)
