@@ -18,22 +18,24 @@
 
 #include <sstream>
 
-#include <gz/common/Console.hh>
-#include <gz/common/Material.hh>
-#include <gz/common/MeshManager.hh>
-#include <gz/common/Skeleton.hh>
-#include <gz/common/SubMesh.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Material.hh>
+#include <ignition/common/MeshManager.hh>
+#include <ignition/common/Skeleton.hh>
+#include <ignition/common/SkeletonAnimation.hh>
+#include <ignition/common/SubMesh.hh>
 
-#include <gz/math/Matrix4.hh>
+#include <ignition/math/Matrix4.hh>
 
-#include "gz/rendering/ogre/OgreMeshFactory.hh"
-#include "gz/rendering/ogre/OgreRenderEngine.hh"
-#include "gz/rendering/ogre/OgreMesh.hh"
-#include "gz/rendering/ogre/OgreScene.hh"
-#include "gz/rendering/ogre/OgreStorage.hh"
-#include "gz/rendering/ogre/OgreIncludes.hh"
+#include "ignition/rendering/ogre/OgreConversions.hh"
+#include "ignition/rendering/ogre/OgreIncludes.hh"
+#include "ignition/rendering/ogre/OgreMesh.hh"
+#include "ignition/rendering/ogre/OgreMeshFactory.hh"
+#include "ignition/rendering/ogre/OgreRenderEngine.hh"
+#include "ignition/rendering/ogre/OgreScene.hh"
+#include "ignition/rendering/ogre/OgreStorage.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -119,8 +121,8 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
     group = Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME;
     ogreMesh = Ogre::MeshManager::getSingleton().createManual(name, group);
 
+    // load skeleton
     Ogre::SkeletonPtr ogreSkeleton;
-
     if (_desc.mesh->HasSkeleton())
     {
       common::SkeletonPtr skel = _desc.mesh->MeshSkeleton();
@@ -129,6 +131,7 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
         Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
         true);
 
+      // load bones
       for (unsigned int i = 0; i < skel->NodeCount(); i++)
       {
         common::SkeletonNode *node = skel->NodeByHandle(i);
@@ -146,9 +149,61 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
         bone->setManuallyControlled(true);
         bone->setInitialState();
       }
+
+      // load skeletal animations
+      for (unsigned int i = 0; i < skel->AnimationCount(); ++i)
+      {
+        common::SkeletonAnimation *skelAnim = skel->Animation(i);
+        if (ogreSkeleton->hasAnimation(skelAnim->Name()))
+        {
+          continue;
+        }
+
+        Ogre::Animation *ogreAnim = ogreSkeleton->createAnimation(
+            skelAnim->Name(), skelAnim->Length());
+
+        for (unsigned int j = 0; j < skel->NodeCount(); ++j)
+        {
+          common::SkeletonNode *node = skel->NodeByHandle(j);
+          common::NodeAnimation *nodeAnim = skelAnim->NodeAnimationByName(
+              node->Name());
+          if (!nodeAnim)
+          {
+            continue;
+          }
+          Ogre::Bone *bone = ogreSkeleton->getBone(node->Name());
+          Ogre::NodeAnimationTrack *ogreNodeAnimTrack =
+              ogreAnim->createNodeTrack(j, bone);
+
+          // set up transform (needed for bvh)
+          math::Matrix4d alignTrans = skel->AlignTranslation(i, node->Name());
+          math::Matrix4d alignRot = skel->AlignRotation(i, node->Name());
+
+          for (unsigned int k = 0; k < nodeAnim->FrameCount(); ++k)
+          {
+            std::pair<double, math::Matrix4d> keyFrame = nodeAnim->KeyFrame(k);
+            Ogre::TransformKeyFrame *kf =
+                ogreNodeAnimTrack->createNodeKeyFrame(keyFrame.first);
+
+            math::Matrix4d p = keyFrame.second;
+
+            // apply anim-skin transform
+            p = alignTrans * p * alignRot;
+
+            auto rot = bone->getOrientation().Inverse() *
+                OgreConversions::Convert(p.Rotation());
+            auto pos = OgreConversions::Convert(p.Translation()) -
+                bone->getPosition();
+            kf->setRotation(rot);
+            kf->setTranslate(pos);
+          }
+        }
+      }
+
       ogreMesh->setSkeletonName(_desc.mesh->Name() + "_skeleton");
     }
 
+    // load submeshes
     for (unsigned int i = 0; i < _desc.mesh->SubMeshCount(); i++)
     {
       auto s = _desc.mesh->SubMeshByIndex(i).lock();
@@ -226,7 +281,7 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
       // see `https://ogrecave.github.io/ogre/api/1.11/_animation.html` under,
       // `Vertex buffer arrangements`.
       currOffset = 0;
-      if (subMesh.TexCoordCount() > 0)
+      if (subMesh.TexCoordCountBySet(0) > 0)
       {
         vertexDecl->addElement(1, currOffset, Ogre::VET_FLOAT2,
             Ogre::VES_TEXTURE_COORDINATES, 0);
@@ -242,7 +297,7 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
                  Ogre::HardwareBuffer::HBU_STATIC_WRITE_ONLY,
                  false);
 
-      if (subMesh.TexCoordCount() > 0)
+      if (subMesh.TexCoordCountBySet(0) > 0)
       {
         texBuf = Ogre::HardwareBufferManager::getSingleton().createVertexBuffer(
             vertexDecl->getVertexSize(1),
@@ -255,7 +310,7 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
       vertices = static_cast<float *>(vBuf->lock(
                       Ogre::HardwareBuffer::HBL_DISCARD));
 
-      if (subMesh.TexCoordCount() > 0)
+      if (subMesh.TexCoordCountBySet(0) > 0)
       {
         vertexData->vertexBufferBinding->setBinding(1, texBuf);
         texMappings = static_cast<float *>(texBuf->lock(
@@ -323,10 +378,10 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
           *vertices++ = subMesh.Normal(j).Z();
         }
 
-        if (subMesh.TexCoordCount() > 0)
+        if (subMesh.TexCoordCountBySet(0) > 0)
         {
-          *texMappings++ = subMesh.TexCoord(j).X();
-          *texMappings++ = subMesh.TexCoord(j).Y();
+          *texMappings++ = subMesh.TexCoordBySet(j, 0).X();
+          *texMappings++ = subMesh.TexCoordBySet(j, 0).Y();
         }
       }
 
@@ -395,6 +450,15 @@ bool OgreMeshFactory::LoadImpl(const MeshDescriptor &_desc)
     return false;
   }
 
+  if (ogreMesh->getNumSubMeshes() == 0u)
+  {
+    std::string msg = "Unable to load mesh: '" + _desc.meshName + "'";
+    if (!_desc.subMeshName.empty())
+      msg += ", submesh: '" + _desc.subMeshName + "'";
+    msg += ". Mesh will be empty.";
+    ignwarn << msg << std::endl;
+  }
+
   return true;
 }
 
