@@ -14,9 +14,17 @@
  * limitations under the License.
  *
  */
-#include "gz/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
 
-using namespace gz;
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <OgreMatrix4.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
+
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
