@@ -24,19 +24,21 @@
 #endif
 
 #include <math.h>
-#include <gz/math/Helpers.hh>
+#include <ignition/math/Helpers.hh>
 
-#include "gz/rendering/RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Conversions.hh"
-#include "gz/rendering/ogre2/Ogre2DepthCamera.hh"
-#include "gz/rendering/ogre2/Ogre2GaussianNoisePass.hh"
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTarget.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
-#include "gz/rendering/ogre2/Ogre2Sensor.hh"
+#include "ignition/rendering/RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2DepthCamera.hh"
+#include "ignition/rendering/ogre2/Ogre2GaussianNoisePass.hh"
+#include "ignition/rendering/ogre2/Ogre2Includes.hh"
+#include "ignition/rendering/ogre2/Ogre2ParticleEmitter.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTarget.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/ogre2/Ogre2Sensor.hh"
 
+#include "Ogre2ParticleNoiseListener.hh"
 
 namespace ignition
 {
@@ -72,7 +74,7 @@ class Ogre2DepthGaussianNoisePass : public Ogre2GaussianNoisePass
 
 /// \internal
 /// \brief Private data for the Ogre2DepthCamera class
-class gz::rendering::Ogre2DepthCameraPrivate
+class ignition::rendering::Ogre2DepthCameraPrivate
 {
   /// \brief The depth buffer
   public: float *depthBuffer = nullptr;
@@ -84,10 +86,10 @@ class gz::rendering::Ogre2DepthCameraPrivate
   public: float *pointCloudImage = nullptr;
 
   /// \brief maximum value used for data outside sensor range
-  public: float dataMaxVal = gz::math::INF_D;
+  public: float dataMaxVal = ignition::math::INF_D;
 
   /// \brief minimum value used for data outside sensor range
-  public: float dataMinVal = -gz::math::INF_D;
+  public: float dataMinVal = -ignition::math::INF_D;
 
   /// \brief 1st pass compositor workspace definition
   public: std::string ogreCompositorWorkspaceDef;
@@ -102,7 +104,7 @@ class gz::rendering::Ogre2DepthCameraPrivate
   public: Ogre::CompositorWorkspace *ogreCompositorWorkspace = nullptr;
 
   /// \brief Output texture with depth and color data
-  public: Ogre::TexturePtr ogreDepthTexture;
+  public: Ogre::TexturePtr ogreDepthTexture[2];
 
   /// \brief Dummy render texture for the depth data
   public: RenderTexturePtr depthTexture;
@@ -120,17 +122,31 @@ class gz::rendering::Ogre2DepthCameraPrivate
   public: bool renderPassDirty = false;
 
   /// \brief Event used to signal rgb point cloud data
-  public: gz::common::EventT<void(const float *,
+  public: ignition::common::EventT<void(const float *,
               unsigned int, unsigned int, unsigned int,
               const std::string &)> newRgbPointCloud;
 
   /// \brief Event used to signal depth data
-  public: gz::common::EventT<void(const float *,
+  public: ignition::common::EventT<void(const float *,
               unsigned int, unsigned int, unsigned int,
               const std::string &)> newDepthFrame;
+
+  /// \brief standard deviation of particle noise
+  public: double particleStddev = 0.01;
+
+  /// \brief Listener for setting particle noise value based on particle
+  /// emitter region
+  public: std::unique_ptr<Ogre2ParticleNoiseListener> particleNoiseListener;
+
+  /// \brief Particle scatter ratio. This is used to determine the ratio of
+  /// particles that will detected by the depth camera
+  public: double particleScatterRatio = 0.1;
+
+  /// \brief Name of sky box material
+  public: const std::string kSkyboxMaterialName = "SkyBox";
 };
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -145,9 +161,9 @@ void Ogre2DepthGaussianNoisePass::PreRender()
   if (!this->enabled)
     return;
 
-  Ogre::Vector3 offsets(math::Rand::DblUniform(0.0, 1.0),
-                        math::Rand::DblUniform(0.0, 1.0),
-                        math::Rand::DblUniform(0.0, 1.0));
+  Ogre::Vector3 offsets(ignition::math::Rand::DblUniform(0.0, 1.0),
+                        ignition::math::Rand::DblUniform(0.0, 1.0),
+                        ignition::math::Rand::DblUniform(0.0, 1.0));
 
   Ogre::Pass *pass = this->gaussianNoiseMat->getTechnique(0)->getPass(0);
   Ogre::GpuProgramParametersSharedPtr psParams =
@@ -281,13 +297,17 @@ void Ogre2DepthCamera::Destroy()
   Ogre::CompositorManager2 *ogreCompMgr = ogreRoot->getCompositorManager2();
 
   // remove depth texture, material, compositor
-  if (this->dataPtr->ogreDepthTexture)
+  for( size_t i = 0u; i < 2u; ++i )
   {
-    Ogre::TextureManager::getSingleton().remove(
-        this->dataPtr->ogreDepthTexture->getName());
+    if (this->dataPtr->ogreDepthTexture[i])
+    {
+      Ogre::TextureManager::getSingleton().remove(
+            this->dataPtr->ogreDepthTexture[i]->getName());
+    }
   }
   if (this->dataPtr->ogreCompositorWorkspace)
   {
+    this->RemoveWorkspaceCrashWorkaround();
     ogreCompMgr->removeWorkspace(
         this->dataPtr->ogreCompositorWorkspace);
   }
@@ -421,6 +441,8 @@ void Ogre2DepthCamera::CreateDepthTexture()
     this->Scene()->BackgroundColor().G(),
     this->Scene()->BackgroundColor().B());
   psParams->setNamedConstant("backgroundColor", bg);
+  psParams->setNamedConstant("particleStddev",
+    static_cast<float>(this->dataPtr->particleStddev));
 
   std::string matDepthFinalName = "DepthCameraFinal";
   Ogre::MaterialPtr matDepthFinal =
@@ -440,6 +462,34 @@ void Ogre2DepthCamera::CreateDepthTexture()
       static_cast<float>(this->dataPtr->dataMaxVal));
   psParamsFinal->setNamedConstant("min",
       static_cast<float>(this->dataPtr->dataMinVal));
+
+  // create background material is specified
+  MaterialPtr backgroundMaterial = this->Scene()->BackgroundMaterial();
+  bool validBackground = backgroundMaterial &&
+      !backgroundMaterial->EnvironmentMap().empty();
+
+  if (validBackground)
+  {
+    Ogre::MaterialManager &matManager = Ogre::MaterialManager::getSingleton();
+    std::string skyMatName = this->dataPtr->kSkyboxMaterialName + "_"
+        + this->Name();
+    auto mat = matManager.getByName(skyMatName);
+    if (!mat)
+    {
+      auto skyboxMat = matManager.getByName(this->dataPtr->kSkyboxMaterialName);
+      if (!skyboxMat)
+      {
+        ignerr << "Unable to find skybox material" << std::endl;
+        return;
+      }
+      mat = skyboxMat->clone(skyMatName);
+    }
+    Ogre::TextureUnitState *texUnit =
+        mat->getTechnique(0u)->getPass(0u)->getTextureUnitState(0u);
+    texUnit->setTextureName(backgroundMaterial->EnvironmentMap(),
+        Ogre::TEX_TYPE_CUBE_MAP);
+  }
+
   // Create depth camera compositor
   auto engine = Ogre2RenderEngine::Instance();
   auto ogreRoot = engine->OgreRoot();
@@ -449,6 +499,22 @@ void Ogre2DepthCamera::CreateDepthTexture()
   this->dataPtr->ogreCompositorWorkspaceDef = wsDefName;
   if (!ogreCompMgr->hasWorkspaceDefinition(wsDefName))
   {
+    // The depth camera compositor does a few passes in order to simulate
+    // particles effects in depth / point cloud image data
+    //
+    // render scene (color) with particles, c1
+    // render scene (depth) without particles, d1
+    // render scene (grayscale) with particles only, g2
+    // render scene (depth) with particles only, d2
+    //
+    //   if g2 is non-zero // pixel with particle
+    //     if d2 < d1 // particle is in view
+    //       apply noise and scatterbility to d2
+    //       set depth data to d2
+    //     else
+    //       set depth data to d1
+    //   set color data to c1
+
     // We need to programmatically create the compositor because we need to
     // configure it to use the cloned depth material created earlier.
     // The compositor node definition is equivalent to the following:
@@ -457,10 +523,12 @@ void Ogre2DepthCamera::CreateDepthTexture()
     // {
     //   texture rt0 target_width target_height PF_FLOAT32_RGBA
     //   texture rt1 target_width target_height PF_FLOAT32_RGBA
-    //   // colorTexture shares same depth buffer as depthTexture
     //   texture colorTexture target_width target_height PF_R8G8B8
     //       depth_texture depth_format PF_D32_FLOAT
     //   texture depthTexture target_width target_height PF_D32_FLOAT
+    //   texture particleTexture target_width target_height PF_L8
+    //   // particleDepthTexture shares same depth buffer as particleTexture
+    //   texture particleDepthTexture target_width target_height PF_D32_FLOAT
     //   target colorTexture
     //   {
     //     pass clear
@@ -471,6 +539,37 @@ void Ogre2DepthCamera::CreateDepthTexture()
     //     {
     //     }
     //   }
+    //   target depthTexture
+    //   {
+    //     pass clear
+    //     {
+    //       colour_value 0.0 0.0 0.0 1.0
+    //     }
+    //     pass render_scene
+    //     {
+    //       visibility_mask 0x11011111
+    //     }
+    //   }
+    //   target particleTexture
+    //   {
+    //     pass clear
+    //     {
+    //     }
+    //     pass render_scene
+    //     {
+    //       visibility_mask 0x00100000
+    //     }
+    //   }
+    //   target particleDepthTexture
+    //   {
+    //     pass clear
+    //     {
+    //     }
+    //     pass render_scene
+    //     {
+    //       visibility_mask 0x00100000
+    //     }
+    //   }
     //   target rt0
     //   {
     //     pass clear
@@ -492,41 +591,11 @@ void Ogre2DepthCamera::CreateDepthTexture()
     this->dataPtr->ogreCompositorBaseNodeDef = baseNodeDefName;
     Ogre::CompositorNodeDef *baseNodeDef =
         ogreCompMgr->addNodeDefinition(baseNodeDefName);
-    Ogre::TextureDefinitionBase::TextureDefinition *rt0TexDef =
-        baseNodeDef->addTextureDefinition("rt0");
-    rt0TexDef->textureType = Ogre::TEX_TYPE_2D;
-    rt0TexDef->width = 0;
-    rt0TexDef->height = 0;
-    rt0TexDef->depth = 1;
-    rt0TexDef->numMipmaps = 0;
-    rt0TexDef->widthFactor = 1;
-    rt0TexDef->heightFactor = 1;
-    rt0TexDef->formatList = {Ogre::PF_FLOAT32_RGBA};
-    rt0TexDef->fsaa = 0;
-    rt0TexDef->uav = false;
-    rt0TexDef->automipmaps = false;
-    rt0TexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
-    rt0TexDef->depthBufferId = Ogre::DepthBuffer::POOL_INVALID;
-    rt0TexDef->depthBufferFormat = Ogre::PF_UNKNOWN;
-    rt0TexDef->fsaaExplicitResolve = false;
 
-    Ogre::TextureDefinitionBase::TextureDefinition *rt1TexDef =
-        baseNodeDef->addTextureDefinition("rt1");
-    rt1TexDef->textureType = Ogre::TEX_TYPE_2D;
-    rt1TexDef->width = 0;
-    rt1TexDef->height = 0;
-    rt1TexDef->depth = 1;
-    rt1TexDef->numMipmaps = 0;
-    rt1TexDef->widthFactor = 1;
-    rt1TexDef->heightFactor = 1;
-    rt1TexDef->formatList = {Ogre::PF_FLOAT32_RGBA};
-    rt1TexDef->fsaa = 0;
-    rt1TexDef->uav = false;
-    rt1TexDef->automipmaps = false;
-    rt1TexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
-    rt1TexDef->depthBufferId = Ogre::DepthBuffer::POOL_INVALID;
-    rt1TexDef->depthBufferFormat = Ogre::PF_UNKNOWN;
-    rt1TexDef->fsaaExplicitResolve = false;
+    baseNodeDef->addTextureSourceName(
+          "rt0", 0u, Ogre::TextureDefinitionBase::TEXTURE_INPUT);
+    baseNodeDef->addTextureSourceName(
+          "rt1", 1u, Ogre::TextureDefinitionBase::TEXTURE_INPUT);
 
     Ogre::TextureDefinitionBase::TextureDefinition *depthTexDef =
         baseNodeDef->addTextureDefinition("depthTexture");
@@ -568,10 +637,52 @@ void Ogre2DepthCamera::CreateDepthTexture()
     colorTexDef->preferDepthTexture = true;
     colorTexDef->fsaaExplicitResolve = false;
 
-    baseNodeDef->setNumTargetPass(2);
+    Ogre::TextureDefinitionBase::TextureDefinition *particleTexDef =
+        baseNodeDef->addTextureDefinition("particleTexture");
+    particleTexDef->textureType = Ogre::TEX_TYPE_2D;
+    particleTexDef->width = 0;
+    particleTexDef->height = 0;
+    particleTexDef->depth = 1;
+    particleTexDef->numMipmaps = 0;
+    particleTexDef->widthFactor = 0.5;
+    particleTexDef->heightFactor = 0.5;
+    particleTexDef->formatList = {Ogre::PF_L8};
+    particleTexDef->fsaa = 0;
+    particleTexDef->uav = false;
+    particleTexDef->automipmaps = false;
+    particleTexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
+    particleTexDef->depthBufferId = Ogre::DepthBuffer::POOL_DEFAULT;
+    particleTexDef->depthBufferFormat = Ogre::PF_UNKNOWN;
+    particleTexDef->preferDepthTexture = false;
+    particleTexDef->fsaaExplicitResolve = false;
+
+    Ogre::TextureDefinitionBase::TextureDefinition *particleDepthTexDef =
+        baseNodeDef->addTextureDefinition("particleDepthTexture");
+    particleDepthTexDef->textureType = Ogre::TEX_TYPE_2D;
+    particleDepthTexDef->width = 0;
+    particleDepthTexDef->height = 0;
+    particleDepthTexDef->depth = 1;
+    particleDepthTexDef->numMipmaps = 0;
+    particleDepthTexDef->widthFactor = 0.5;
+    particleDepthTexDef->heightFactor = 0.5;
+    particleDepthTexDef->formatList = {Ogre::PF_D32_FLOAT};
+    particleDepthTexDef->fsaa = 0;
+    particleDepthTexDef->uav = false;
+    particleDepthTexDef->automipmaps = false;
+    particleDepthTexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
+    particleDepthTexDef->depthBufferId = Ogre::DepthBuffer::POOL_NON_SHAREABLE;
+    particleDepthTexDef->depthBufferFormat = Ogre::PF_UNKNOWN;
+    particleDepthTexDef->fsaaExplicitResolve = false;
+
+    baseNodeDef->setNumTargetPass(5);
+
     Ogre::CompositorTargetDef *colorTargetDef =
         baseNodeDef->addTargetPass("colorTexture");
-    colorTargetDef->setNumPasses(2);
+
+    if (validBackground)
+      colorTargetDef->setNumPasses(3);
+    else
+      colorTargetDef->setNumPasses(2);
     {
       // clear pass
       Ogre::CompositorPassClearDef *passClear =
@@ -579,19 +690,87 @@ void Ogre2DepthCamera::CreateDepthTexture()
           colorTargetDef->addPass(Ogre::PASS_CLEAR));
       passClear->mColourValue = Ogre::ColourValue(
           Ogre2Conversions::Convert(this->Scene()->BackgroundColor()));
+
+      if (validBackground)
+      {
+        // quad pass
+        Ogre::CompositorPassQuadDef *passQuad =
+            static_cast<Ogre::CompositorPassQuadDef *>(
+            colorTargetDef->addPass(Ogre::PASS_QUAD));
+        passQuad->mMaterialName = this->dataPtr->kSkyboxMaterialName + "_"
+            + this->Name();
+        passQuad->mFrustumCorners =
+            Ogre::CompositorPassQuadDef::CAMERA_DIRECTION;
+      }
+
       // scene pass
       Ogre::CompositorPassSceneDef *passScene =
           static_cast<Ogre::CompositorPassSceneDef *>(
           colorTargetDef->addPass(Ogre::PASS_SCENE));
-      passScene->mVisibilityMask = IGN_VISIBILITY_ALL
-          & ~(IGN_VISIBILITY_GUI | IGN_VISIBILITY_SELECTABLE);
-      // todo(anyone) Fix shadows. The shadow compositor node gets rebuilt
-      // when the number of shadow-casting light changes so we end up with
-      // invalid shadow node here. See Ogre2Scene::PreRender function on how
-      // it destroys and triggers a compositor rebuild in OgreCamera when
-      // the number of shadow-casting light changes
-      // passScene->mShadowNode = "PbsMaterialsShadowNode";
+      passScene->mVisibilityMask = IGN_VISIBILITY_ALL;
+
+      // todo(anyone) PbsMaterialsShadowNode is hardcoded.
+      // Although this may be just fine
+      passScene->mShadowNode = "PbsMaterialsShadowNode";
     }
+
+    Ogre::CompositorTargetDef *depthTargetDef =
+        baseNodeDef->addTargetPass("depthTexture");
+    depthTargetDef->setNumPasses(2);
+    {
+      // clear pass
+      Ogre::CompositorPassClearDef *passClear =
+          static_cast<Ogre::CompositorPassClearDef *>(
+          depthTargetDef->addPass(Ogre::PASS_CLEAR));
+      passClear->mColourValue = Ogre::ColourValue(this->FarClipPlane(),
+          this->FarClipPlane(), this->FarClipPlane());
+
+      // scene pass
+      Ogre::CompositorPassSceneDef *passScene =
+          static_cast<Ogre::CompositorPassSceneDef *>(
+          depthTargetDef->addPass(Ogre::PASS_SCENE));
+      // depth texute does not contain particles
+      passScene->mVisibilityMask = IGN_VISIBILITY_ALL
+          & ~Ogre2ParticleEmitter::kParticleVisibilityFlags;
+    }
+
+    Ogre::CompositorTargetDef *particleTargetDef =
+        baseNodeDef->addTargetPass("particleTexture");
+    particleTargetDef->setNumPasses(2);
+    {
+      // clear pass
+      Ogre::CompositorPassClearDef *passClear =
+          static_cast<Ogre::CompositorPassClearDef *>(
+          particleTargetDef->addPass(Ogre::PASS_CLEAR));
+      passClear->mColourValue = Ogre::ColourValue::Black;
+
+      // scene pass
+      Ogre::CompositorPassSceneDef *passScene =
+          static_cast<Ogre::CompositorPassSceneDef *>(
+          particleTargetDef->addPass(Ogre::PASS_SCENE));
+      passScene->mVisibilityMask =
+          Ogre2ParticleEmitter::kParticleVisibilityFlags;
+    }
+
+    Ogre::CompositorTargetDef *particleDepthTargetDef =
+        baseNodeDef->addTargetPass("particleDepthTexture");
+    particleDepthTargetDef->setNumPasses(2);
+    {
+      // clear pass
+      Ogre::CompositorPassClearDef *passClear =
+          static_cast<Ogre::CompositorPassClearDef *>(
+          particleDepthTargetDef->addPass(Ogre::PASS_CLEAR));
+      passClear->mColourValue = Ogre::ColourValue(this->FarClipPlane(),
+          this->FarClipPlane(), this->FarClipPlane());
+
+      // scene pass
+      Ogre::CompositorPassSceneDef *passScene =
+          static_cast<Ogre::CompositorPassSceneDef *>(
+          particleDepthTargetDef->addPass(Ogre::PASS_SCENE));
+      passScene->mVisibilityMask =
+          Ogre2ParticleEmitter::kParticleVisibilityFlags;
+    }
+
     // rt0 target - converts depth to xyz
     Ogre::CompositorTargetDef *inTargetDef =
         baseNodeDef->addTargetPass("rt0");
@@ -611,6 +790,8 @@ void Ogre2DepthCamera::CreateDepthTexture()
       passQuad->mMaterialName = this->dataPtr->depthMaterial->getName();
       passQuad->addQuadTextureSource(0, "depthTexture", 0);
       passQuad->addQuadTextureSource(1, "colorTexture", 0);
+      passQuad->addQuadTextureSource(2, "particleTexture", 0);
+      passQuad->addQuadTextureSource(3, "particleDepthTexture", 0);
       passQuad->mFrustumCorners =
           Ogre::CompositorPassQuadDef::VIEW_SPACE_CORNERS;
     }
@@ -645,10 +826,10 @@ void Ogre2DepthCamera::CreateDepthTexture()
     Ogre::CompositorNodeDef *finalNodeDef =
         ogreCompMgr->addNodeDefinition(finalNodeDefName);
 
-    // output texture
-    finalNodeDef->addTextureSourceName("rt_output", 0,
+    finalNodeDef->addTextureSourceName("rt_input", 0,
         Ogre::TextureDefinitionBase::TEXTURE_INPUT);
-    finalNodeDef->addTextureSourceName("rt_input", 1,
+    // output texture
+    finalNodeDef->addTextureSourceName("rt_output", 1,
         Ogre::TextureDefinitionBase::TEXTURE_INPUT);
 
     finalNodeDef->setNumTargetPass(1);
@@ -684,8 +865,9 @@ void Ogre2DepthCamera::CreateDepthTexture()
     Ogre::CompositorWorkspaceDef *workDef =
         ogreCompMgr->addWorkspaceDefinition(wsDefName);
 
-    workDef->connect(baseNodeDefName, 0,  finalNodeDefName, 1);
-    workDef->connectExternal(0, finalNodeDefName, 0);
+    workDef->connectExternal(0, baseNodeDefName, 0);
+    workDef->connectExternal(1, baseNodeDefName, 1);
+    workDef->connect(baseNodeDefName, finalNodeDefName);
   }
   Ogre::CompositorWorkspaceDef *wsDef =
       ogreCompMgr->getWorkspaceDefinition(wsDefName);
@@ -697,20 +879,62 @@ void Ogre2DepthCamera::CreateDepthTexture()
   }
 
   // create render texture - these textures pack the range data
-  this->dataPtr->ogreDepthTexture =
-    Ogre::TextureManager::getSingleton().createManual(
-    this->Name() + "_depth", "General", Ogre::TEX_TYPE_2D,
-    this->ImageWidth(), this->ImageHeight(), 1, 0,
-    Ogre::PF_FLOAT32_RGBA, Ogre::TU_RENDERTARGET,
-    0, false, 0, Ogre::BLANKSTRING, false, true);
+  for( size_t i = 0u; i < 2u; ++i )
+  {
+    this->dataPtr->ogreDepthTexture[i] =
+      Ogre::TextureManager::getSingleton().createManual(
+      this->Name() + "_depth" + std::to_string(i), "General",
+      Ogre::TEX_TYPE_2D, this->ImageWidth(), this->ImageHeight(), 1, 0,
+      Ogre::PF_FLOAT32_RGBA, Ogre::TU_RENDERTARGET,
+      0, false, 0, Ogre::BLANKSTRING, false, true);
 
-  Ogre::RenderTarget *rt =
-    this->dataPtr->ogreDepthTexture->getBuffer()->getRenderTarget();
+    Ogre::RenderTarget *rt =
+        this->dataPtr->ogreDepthTexture[i]->getBuffer()->getRenderTarget();
+    rt->setDepthBufferPool(Ogre::DepthBuffer::POOL_INVALID);
+  }
+
+  CreateWorkspaceInstance();
+}
+
+//////////////////////////////////////////////////
+void Ogre2DepthCamera::CreateWorkspaceInstance()
+{
+  auto engine = Ogre2RenderEngine::Instance();
+  auto ogreRoot = engine->OgreRoot();
+  Ogre::CompositorManager2 *ogreCompMgr = ogreRoot->getCompositorManager2();
+
+  Ogre::CompositorChannelVec externalTargets(2u);
+  for( size_t i = 0u; i < 2u; ++i )
+  {
+    externalTargets[i].target =
+        this->dataPtr->ogreDepthTexture[i]->getBuffer()->getRenderTarget();
+    externalTargets[i].textures.push_back(this->dataPtr->ogreDepthTexture[i]);
+  }
 
   // create compositor worksspace
   this->dataPtr->ogreCompositorWorkspace =
       ogreCompMgr->addWorkspace(this->scene->OgreSceneManager(),
-      rt, this->ogreCamera, wsDefName, false);
+      externalTargets, this->ogreCamera,
+      this->dataPtr->ogreCompositorWorkspaceDef, false);
+
+  // add the listener
+  Ogre::CompositorNode *node =
+      this->dataPtr->ogreCompositorWorkspace->getNodeSequence()[0];
+  auto channelsTex = node->getLocalTextures();
+
+  for (auto c : channelsTex)
+  {
+    if (c.textures[0]->getSrcFormat() == Ogre::PF_L8)
+    {
+      // add particle noise / scatter effects listener so we can set the
+      // amount of noise based on size of emitter
+      this->dataPtr->particleNoiseListener.reset(
+          new Ogre2ParticleNoiseListener(this->scene,
+          this->ogreCamera, this->dataPtr->depthMaterial));
+      c.target->addListener(this->dataPtr->particleNoiseListener.get());
+      break;
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -726,9 +950,18 @@ void Ogre2DepthCamera::Render()
 //////////////////////////////////////////////////
 void Ogre2DepthCamera::PreRender()
 {
-  if (!this->dataPtr->ogreDepthTexture)
+  if (!this->dataPtr->ogreDepthTexture[0])
     this->CreateDepthTexture();
 
+  if (!this->dataPtr->ogreCompositorWorkspace)
+    this->CreateWorkspaceInstance();
+
+  Ogre::Texture *rawDepthTextures[2] =
+  {
+    this->dataPtr->ogreDepthTexture[0].get(),
+    this->dataPtr->ogreDepthTexture[1].get()
+  };
+
   // update depth camera render passes
   Ogre2RenderTarget::UpdateRenderPassChain(
       this->dataPtr->ogreCompositorWorkspace,
@@ -736,9 +969,43 @@ void Ogre2DepthCamera::PreRender()
       this->dataPtr->ogreCompositorBaseNodeDef,
       this->dataPtr->ogreCompositorFinalNodeDef,
       this->dataPtr->renderPasses,
-      this->dataPtr->renderPassDirty);
+      this->dataPtr->renderPassDirty,
+      &rawDepthTextures,
+      false);
+
+  if (rawDepthTextures[0] != this->dataPtr->ogreDepthTexture[0].get())
+  {
+    std::swap( this->dataPtr->ogreDepthTexture[0],
+               this->dataPtr->ogreDepthTexture[1] );
+  }
+
   for (auto &pass : this->dataPtr->renderPasses)
     pass->PreRender();
+
+
+  // add the particle noise listener again if worksapce is recreated due to
+  // dirty render pass
+  if (this->dataPtr->renderPassDirty)
+  {
+    Ogre::CompositorNode *node =
+        this->dataPtr->ogreCompositorWorkspace->getNodeSequence()[0];
+    auto channelsTex = node->getLocalTextures();
+
+    for (auto c : channelsTex)
+    {
+      if (c.textures[0]->getSrcFormat() == Ogre::PF_L8)
+      {
+        // add particle noise / scatter effects listener so we can set the
+        // amount of noise based on size of emitter
+        this->dataPtr->particleNoiseListener.reset(
+            new Ogre2ParticleNoiseListener(this->scene,
+            this->ogreCamera, this->dataPtr->depthMaterial));
+        c.target->addListener(this->dataPtr->particleNoiseListener.get());
+        break;
+      }
+    }
+  }
+
   this->dataPtr->renderPassDirty = false;
 }
 
@@ -763,7 +1030,7 @@ void Ogre2DepthCamera::PostRender()
         1, imageFormat, this->dataPtr->depthBuffer);
 
   // blit data from gpu to cpu
-  auto rt = this->dataPtr->ogreDepthTexture->getBuffer()->getRenderTarget();
+  auto rt = this->dataPtr->ogreDepthTexture[1]->getBuffer()->getRenderTarget();
   rt->copyContentsToMemory(dstBox, Ogre::RenderTarget::FB_AUTO);
 
   if (!this->dataPtr->depthImage)
@@ -847,7 +1114,7 @@ const float *Ogre2DepthCamera::DepthData() const
 }
 
 //////////////////////////////////////////////////
-common::ConnectionPtr Ogre2DepthCamera::ConnectNewDepthFrame(
+ignition::common::ConnectionPtr Ogre2DepthCamera::ConnectNewDepthFrame(
     std::function<void(const float *, unsigned int, unsigned int,
       unsigned int, const std::string &)>  _subscriber)
 {
@@ -855,7 +1122,7 @@ common::ConnectionPtr Ogre2DepthCamera::ConnectNewDepthFrame(
 }
 
 //////////////////////////////////////////////////
-common::ConnectionPtr Ogre2DepthCamera::ConnectNewRgbPointCloud(
+ignition::common::ConnectionPtr Ogre2DepthCamera::ConnectNewRgbPointCloud(
     std::function<void(const float *, unsigned int, unsigned int,
       unsigned int, const std::string &)>  _subscriber)
 {
@@ -900,6 +1167,38 @@ double Ogre2DepthCamera::FarClipPlane() const
   return BaseDepthCamera::FarClipPlane();
 }
 
+//////////////////////////////////////////////////
+void Ogre2DepthCamera::SetShadowsNodeDefDirty()
+{
+  if (!this->dataPtr->ogreCompositorWorkspace)
+    return;
+
+  auto engine = Ogre2RenderEngine::Instance();
+  auto ogreRoot = engine->OgreRoot();
+  Ogre::CompositorManager2 *ogreCompMgr = ogreRoot->getCompositorManager2();
+
+  this->RemoveWorkspaceCrashWorkaround();
+  ogreCompMgr->removeWorkspace( this->dataPtr->ogreCompositorWorkspace );
+  this->dataPtr->ogreCompositorWorkspace = nullptr;
+}
+
+//////////////////////////////////////////////////
+void Ogre2DepthCamera::RemoveWorkspaceCrashWorkaround()
+{
+  Ogre::MaterialPtr material =
+      Ogre::MaterialManager::getSingleton().
+      getByName (this->dataPtr->depthMaterial->getName());
+
+  if (!material.isNull())
+  {
+    for (size_t i = 0; i < 4; ++i)
+    {
+      material->getBestTechnique()->getPass(0)->
+          getTextureUnitState(i)->setBlank();
+    }
+  }
+}
+
 //////////////////////////////////////////////////
 void Ogre2DepthCamera::AddRenderPass(const RenderPassPtr &_pass)
 {
