@@ -17,30 +17,41 @@
 
 // Note this include is placed in the src file because
 // otherwise ogre produces compile errors
-#if defined(_MSC_VER)
-#pragma warning(push)
-#pragma warning(disable: 5033)
+#ifdef _MSC_VER
+#pragma warning(push, 0)
 #endif
 #include <Hlms/Pbs/OgreHlmsPbsDatablock.h>
-#if defined(_MSC_VER)
+#ifdef _MSC_VER
 #pragma warning(pop)
 #endif
 
-#include "gz/common/Console.hh"
-#include "gz/rendering/ogre2/Ogre2Conversions.hh"
-#include "gz/rendering/ogre2/Ogre2DynamicRenderable.hh"
-#include "gz/rendering/ogre2/Ogre2Material.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/common/Console.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2DynamicRenderable.hh"
+#include "ignition/rendering/ogre2/Ogre2Material.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <OgreItem.h>
+#include <OgreMesh2.h>
+#include <OgreMeshManager2.h>
+#include <OgreSceneManager.h>
+#include <OgreSubMesh2.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
 
 /// \brief Private implementation
-class gz::rendering::Ogre2DynamicRenderablePrivate
+class ignition::rendering::Ogre2DynamicRenderablePrivate
 {
   /// \brief list of colors at each point
-  public: std::vector<gz::math::Color> colors;
+  public: std::vector<ignition::math::Color> colors;
 
   /// \brief List of vertices for the mesh
-  public: std::vector<gz::math::Vector3d> vertices;
+  public: std::vector<ignition::math::Vector3d> vertices;
 
   /// \brief Used to indicate if the lines require an update
   public: bool dirty = false;
@@ -81,7 +92,7 @@ class gz::rendering::Ogre2DynamicRenderablePrivate
 };
 
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -132,8 +143,8 @@ void Ogre2DynamicRenderable::DestroyBuffer()
   if (this->dataPtr->vbuffer)
     delete [] this->dataPtr->vbuffer;
 
-  Ogre::Root *root = Ogre2RenderEngine::Instance()->OgreRoot();
-  Ogre::RenderSystem *renderSystem = root->getRenderSystem();
+  Ogre::RenderSystem *renderSystem =
+      this->dataPtr->sceneManager->getDestinationRenderSystem();
   Ogre::VaoManager *vaoManager = renderSystem->getVaoManager();
 
   if (!vaoManager)
@@ -188,8 +199,8 @@ void Ogre2DynamicRenderable::UpdateBuffer()
   if (!this->dataPtr->dirty)
     return;
 
-  Ogre::Root *root = Ogre2RenderEngine::Instance()->OgreRoot();
-  Ogre::RenderSystem *renderSystem = root->getRenderSystem();
+  Ogre::RenderSystem *renderSystem =
+      this->dataPtr->sceneManager->getDestinationRenderSystem();
 
   Ogre::VaoManager *vaoManager = renderSystem->getVaoManager();
   if (!vaoManager)
@@ -405,8 +416,8 @@ MarkerType Ogre2DynamicRenderable::OperationType() const
 }
 
 /////////////////////////////////////////////////
-void Ogre2DynamicRenderable::AddPoint(const math::Vector3d &_pt,
-                                      const math::Color &_color)
+void Ogre2DynamicRenderable::AddPoint(const ignition::math::Vector3d &_pt,
+                                      const ignition::math::Color &_color)
 {
   this->dataPtr->vertices.push_back(_pt);
 
@@ -421,14 +432,14 @@ void Ogre2DynamicRenderable::AddPoint(const math::Vector3d &_pt,
 
 /////////////////////////////////////////////////
 void Ogre2DynamicRenderable::AddPoint(double _x, double _y, double _z,
-                                      const math::Color &_color)
+                                      const ignition::math::Color &_color)
 {
-  this->AddPoint(math::Vector3d(_x, _y, _z), _color);
+  this->AddPoint(ignition::math::Vector3d(_x, _y, _z), _color);
 }
 
 /////////////////////////////////////////////////
 void Ogre2DynamicRenderable::SetPoint(unsigned int _index,
-                                      const math::Vector3d &_value)
+                                      const ignition::math::Vector3d &_value)
 {
   if (_index >= this->dataPtr->vertices.size())
   {
@@ -444,7 +455,7 @@ void Ogre2DynamicRenderable::SetPoint(unsigned int _index,
 
 /////////////////////////////////////////////////
 void Ogre2DynamicRenderable::SetColor(unsigned int _index,
-                                      const math::Color &_color)
+                                      const ignition::math::Color &_color)
 {
   if (_index >= this->dataPtr->colors.size())
   {
@@ -464,7 +475,7 @@ void Ogre2DynamicRenderable::SetColor(unsigned int _index,
 }
 
 /////////////////////////////////////////////////
-math::Vector3d Ogre2DynamicRenderable::Point(
+ignition::math::Vector3d Ogre2DynamicRenderable::Point(
     const unsigned int _index) const
 {
   if (_index >= this->dataPtr->vertices.size())
@@ -472,9 +483,9 @@ math::Vector3d Ogre2DynamicRenderable::Point(
     ignerr << "Point index[" << _index << "] is out of bounds[0-"
            << this->dataPtr->vertices.size()-1 << "]\n";
 
-    return math::Vector3d(math::INF_D,
-                                    math::INF_D,
-                                    math::INF_D);
+    return ignition::math::Vector3d(ignition::math::INF_D,
+                                    ignition::math::INF_D,
+                                    ignition::math::INF_D);
   }
 
   return this->dataPtr->vertices[_index];
