@@ -16,18 +16,41 @@
  */
 
 
-#include <gz/common/Console.hh>
+#include <ignition/common/Console.hh>
 
-#include "gz/rendering/RenderPassSystem.hh"
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2GaussianNoisePass.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/RenderPassSystem.hh"
+#include "ignition/rendering/ogre2/Ogre2GaussianNoisePass.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
 
-using namespace gz;
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <Compositor/OgreCompositorManager2.h>
+#include <Compositor/OgreCompositorNodeDef.h>
+#include <Compositor/Pass/PassQuad/OgreCompositorPassQuadDef.h>
+#include <OgreMaterial.h>
+#include <OgreMaterialManager.h>
+#include <OgrePass.h>
+#include <OgreRoot.h>
+#include <OgreTechnique.h>
+#include <OgreVector3.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
+
+/// \brief Private data for the Ogre2GaussianNoisePass class
+class ignition::rendering::Ogre2GaussianNoisePassPrivate
+{
+  /// brief Pointer to the Gaussian noise ogre material
+  public: Ogre::Material *gaussianNoiseMat = nullptr;
+};
+
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
 Ogre2GaussianNoisePass::Ogre2GaussianNoisePass()
+  : dataPtr(std::make_unique<Ogre2GaussianNoisePassPrivate>())
 {
 }
 
@@ -39,7 +62,7 @@ Ogre2GaussianNoisePass::~Ogre2GaussianNoisePass()
 //////////////////////////////////////////////////
 void Ogre2GaussianNoisePass::PreRender()
 {
-  if (!this->gaussianNoiseMat)
+  if (!this->dataPtr->gaussianNoiseMat)
     return;
 
   if (!this->enabled)
@@ -51,14 +74,15 @@ void Ogre2GaussianNoisePass::PreRender()
   // Sample three values within the range [0,1.0] and set them for use in
   // the fragment shader, which will interpret them as offsets from (0,0)
   // to use when computing pseudo-random values.
-  Ogre::Vector3 offsets(math::Rand::DblUniform(0.0, 1.0),
-                        math::Rand::DblUniform(0.0, 1.0),
-                        math::Rand::DblUniform(0.0, 1.0));
+  Ogre::Vector3 offsets(ignition::math::Rand::DblUniform(0.0, 1.0),
+                        ignition::math::Rand::DblUniform(0.0, 1.0),
+                        ignition::math::Rand::DblUniform(0.0, 1.0));
   // These calls are setting parameters that are declared in two places:
   // 1. media/materials/scripts/gaussian_noise.material, in
   //    fragment_program GaussianNoiseFS
   // 2. media/materials/scripts/gaussian_noise_fs.glsl
-  Ogre::Pass *pass = this->gaussianNoiseMat->getTechnique(0)->getPass(0);
+  Ogre::Pass *pass =
+      this->dataPtr->gaussianNoiseMat->getTechnique(0)->getPass(0);
   Ogre::GpuProgramParametersSharedPtr psParams =
       pass->getFragmentProgramParameters();
   psParams->setNamedConstant("offsets", offsets);
@@ -97,7 +121,7 @@ void Ogre2GaussianNoisePass::CreateRenderPass()
     ogreMat->load();
   std::string materialName = matName + "_" +
       std::to_string(gaussianNodeCounter);
-  this->gaussianNoiseMat = ogreMat->clone(materialName).get();
+  this->dataPtr->gaussianNoiseMat = ogreMat->clone(materialName).get();
 
   // create the compostior node definition
 
