@@ -15,23 +15,45 @@
  *
 */
 
-#include <gz/math/Vector2.hh>
-#include <gz/math/Vector3.hh>
+#include <ignition/math/Vector2.hh>
+#include <ignition/math/Vector3.hh>
 
-#include <gz/common/Console.hh>
-#include <gz/math/Helpers.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/math/Helpers.hh>
 
-#include "gz/rendering/ogre2/Ogre2Camera.hh"
-#include "gz/rendering/ogre2/Ogre2GpuRays.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Conversions.hh"
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTarget.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
-#include "gz/rendering/ogre2/Ogre2Sensor.hh"
-#include "gz/rendering/ogre2/Ogre2Visual.hh"
+#include "ignition/rendering/ogre2/Ogre2Camera.hh"
+#include "ignition/rendering/ogre2/Ogre2GpuRays.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2ParticleEmitter.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTarget.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/ogre2/Ogre2Sensor.hh"
+#include "ignition/rendering/ogre2/Ogre2Visual.hh"
+
+#include "Ogre2ParticleNoiseListener.hh"
+
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <Compositor/OgreCompositorManager2.h>
+#include <Compositor/OgreCompositorWorkspace.h>
+#include <Compositor/Pass/PassClear/OgreCompositorPassClearDef.h>
+#include <Compositor/Pass/PassQuad/OgreCompositorPassQuadDef.h>
+#include <Compositor/Pass/PassScene/OgreCompositorPassSceneDef.h>
+#include <OgreDepthBuffer.h>
+#include <OgreHardwarePixelBuffer.h>
+#include <OgreItem.h>
+#include <OgreRenderTexture.h>
+#include <OgreRoot.h>
+#include <OgreSceneManager.h>
+#include <OgreTechnique.h>
+#include <OgreTextureManager.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
 
 namespace ignition
 {
@@ -83,7 +105,7 @@ class Ogre2LaserRetroMaterialSwitcher : public Ogre::RenderTargetListener
 
 /// \internal
 /// \brief Private data for the Ogre2GpuRays class
-class gz::rendering::Ogre2GpuRaysPrivate
+class ignition::rendering::Ogre2GpuRaysPrivate
 {
   /// \brief Event triggered when new gpu rays range data are available.
   /// \param[in] _frame New frame containing raw gpu rays data.
@@ -91,7 +113,7 @@ class gz::rendering::Ogre2GpuRaysPrivate
   /// \param[in] _height Height of frame.
   /// \param[in] _channel Number of channels
   /// \param[in] _format Format of frame.
-  public: gz::common::EventT<void(const float *,
+  public: ignition::common::EventT<void(const float *,
                unsigned int, unsigned int, unsigned int,
                const std::string &)> newGpuRaysFrame;
 
@@ -104,7 +126,7 @@ class gz::rendering::Ogre2GpuRaysPrivate
   /// \brief Pointer to Ogre material for the first rendering pass.
   public: Ogre::MaterialPtr matFirstPass;
 
-  /// \brief Pointer to Ogre material for the sencod rendering pass.
+  /// \brief Pointer to Ogre material for the second rendering pass.
   public: Ogre::MaterialPtr matSecondPass;
 
   /// \brief Cubemap cameras
@@ -162,9 +184,19 @@ class gz::rendering::Ogre2GpuRaysPrivate
   /// \brief Pointer to material switcher
   public: std::unique_ptr<Ogre2LaserRetroMaterialSwitcher>
       laserRetroMaterialSwitcher[6];
+
+  /// \brief standard deviation of particle noise
+  public: double particleStddev = 0.01;
+
+  /// \brief Listener for setting particle noise value based on particle
+  /// emitter region
+  public: std::unique_ptr<Ogre2ParticleNoiseListener> particleNoiseListener[6];
+
+  /// \brief Min allowed angle in radians;
+  public: const math::Angle kMinAllowedAngle = 1e-4;
 };
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 
@@ -226,13 +258,13 @@ void Ogre2LaserRetroMaterialSwitcher::preRenderTargetUpdate(
         {
           try
           {
-            retroValue = static_cast<float>(std::get<double>(tempLaserRetro));
+            retroValue = std::get<double>(tempLaserRetro);
           }
           catch(...)
           {
             try
             {
-              retroValue = static_cast<float>(std::get<int>(tempLaserRetro));
+              retroValue = std::get<int>(tempLaserRetro);
             }
             catch(std::bad_variant_access &e)
             {
@@ -444,14 +476,17 @@ void Ogre2GpuRays::CreateCamera()
 void Ogre2GpuRays::ConfigureCamera()
 {
   // horizontal gpu rays setup
-  this->SetHFOV(this->AngleMax() - this->AngleMin());
+  auto hfovAngle = this->AngleMax() - this->AngleMin();
+  hfovAngle = std::max(this->dataPtr->kMinAllowedAngle, hfovAngle);
+  this->SetHFOV(hfovAngle);
 
   // vertical laser setup
   double vfovAngle;
 
   if (this->VerticalRangeCount() > 1)
   {
-    vfovAngle = (this->VerticalAngleMax() - this->VerticalAngleMin()).Radian();
+    vfovAngle = std::max(this->dataPtr->kMinAllowedAngle.Radian(),
+        (this->VerticalAngleMax() - this->VerticalAngleMin()).Radian());
   }
   else
   {
@@ -467,7 +502,31 @@ void Ogre2GpuRays::ConfigureCamera()
   this->SetVFOV(vfovAngle);
 
   // Configure first pass texture size
-  this->Set1stTextureSize(1024, 1024);
+  // Each cubemap texture covers 90 deg FOV so determine number of samples
+  // within the view for both horizontal and vertical FOV
+  unsigned int hs = static_cast<unsigned int>(
+      IGN_PI * 0.5 / hfovAngle.Radian() * this->RangeCount());
+  unsigned int vs = static_cast<unsigned int>(
+      IGN_PI * 0.5 / vfovAngle * this->VerticalRangeCount());
+
+  // get the max number from the two
+  unsigned int v = std::max(hs, vs);
+  // round to next highest power of 2
+  // https://graphics.stanford.edu/~seander/bithacks.html#RoundUpPowerOf2
+  v--;
+  v |= v >> 1;
+  v |= v >> 2;
+  v |= v >> 4;
+  v |= v >> 8;
+  v |= v >> 16;
+  v++;
+  // limit max texture size to 1024
+  unsigned int min1stPassSamples = 2u;
+  unsigned int max1stPassSamples = 1024u;
+  unsigned int samples1stPass =
+      std::clamp(v, min1stPassSamples, max1stPassSamples);
+
+  this->Set1stTextureSize(samples1stPass, samples1stPass);
 
   // Configure second pass texture size
   this->SetRangeCount(this->RangeCount(), this->VerticalRangeCount());
@@ -486,19 +545,19 @@ math::Vector2d Ogre2GpuRays::SampleCubemap(const math::Vector3d &_v,
   math::Vector2d uv;
   if (vAbs.Z() >= vAbs.X() && vAbs.Z() >= vAbs.Y())
   {
-    _faceIndex = _v.Z() < 0.0 ? 5 : 4;
+    _faceIndex = _v.Z() < 0.0 ? 5.0 : 4.0;
     ma = 0.5 / vAbs.Z();
     uv = math::Vector2d(_v.Z() < 0.0 ? -_v.X() : _v.X(), -_v.Y());
   }
   else if (vAbs.Y() >= vAbs.X())
   {
-    _faceIndex = _v.Y() < 0.0 ? 3 : 2;
+    _faceIndex = _v.Y() < 0.0 ? 3.0 : 2.0;
     ma = 0.5 / vAbs.Y();
     uv = math::Vector2d(_v.X(), _v.Y() < 0.0 ? -_v.Z() : _v.Z());
   }
   else
   {
-    _faceIndex = _v.X() < 0.0 ? 1 : 0;
+    _faceIndex = _v.X() < 0.0 ? 1.0 : 0.0;
     ma = 0.5 / vAbs.X();
     uv = math::Vector2d(_v.X() < 0.0 ? _v.Z() : -_v.Z(), -_v.Y());
   }
@@ -512,11 +571,16 @@ void Ogre2GpuRays::CreateSampleTexture()
   double max = this->AngleMax().Radian();
   double vmin = this->VerticalAngleMin().Radian();
   double vmax = this->VerticalAngleMax().Radian();
-  double hStep = (max-min) / static_cast<double>(this->dataPtr->w2nd-1);
+
+  double hAngle = std::max(this->dataPtr->kMinAllowedAngle.Radian(), max - min);
+  double vAngle = std::max(this->dataPtr->kMinAllowedAngle.Radian(),
+      vmax - vmin);
+
+  double hStep = hAngle / static_cast<double>(this->dataPtr->w2nd-1);
   double vStep = 1.0;
   // non-planar case
   if (this->dataPtr->h2nd > 1)
-    vStep = (vmax-vmin) / static_cast<double>(this->dataPtr->h2nd-1);
+    vStep = vAngle / static_cast<double>(this->dataPtr->h2nd-1);
 
   // create an RGB texture (cubeUVTex) to pack info that tells the shaders how
   // to sample from the cubemap textures.
@@ -564,7 +628,7 @@ void Ogre2GpuRays::CreateSampleTexture()
       // v
       *pDest++ = uv.Y();
       // face
-      *pDest++ = static_cast<float>(faceIdx);
+      *pDest++ = faceIdx;
 
        h += hStep;
     }
@@ -607,6 +671,8 @@ void Ogre2GpuRays::Setup1stPass()
       static_cast<float>(this->dataMaxVal));
   psParams->setNamedConstant("min",
       static_cast<float>(this->dataMinVal));
+  psParams->setNamedConstant("particleStddev",
+    static_cast<float>(this->dataPtr->particleStddev));
 
   // Create 1st pass compositor
   auto engine = Ogre2RenderEngine::Instance();
@@ -622,6 +688,8 @@ void Ogre2GpuRays::Setup1stPass()
   //   in 0 rt_input
   //   texture depthTexture target_width target_height PF_D32_FLOAT
   //   texture colorTexture target_width target_height PF_R8G8B8
+  //   texture particleTexture target_width target_height PF_L8
+  //   texture particleDepthTexture target_width target_height PF_D32_FLOAT
   //   target colorTexture
   //   {
   //     pass clear
@@ -630,6 +698,18 @@ void Ogre2GpuRays::Setup1stPass()
   //     }
   //     pass render_scene
   //     {
+  //       visibility_mask 0x11011111
+  //     }
+  //   }
+  //   target particleTexture
+  //   {
+  //     pass clear
+  //     {
+  //       colour_value 0.0 0.0 0.0 1.0
+  //     }
+  //     pass render_scene
+  //     {
+  //       visibility_mask 0.00100000
   //     }
   //   }
   //   target rt_input
@@ -697,7 +777,42 @@ void Ogre2GpuRays::Setup1stPass()
     colorTexDef->preferDepthTexture = true;
     colorTexDef->fsaaExplicitResolve = false;
 
-    nodeDef->setNumTargetPass(2);
+    Ogre::TextureDefinitionBase::TextureDefinition *particleDepthTexDef =
+        nodeDef->addTextureDefinition("particleDepthTexture");
+    particleDepthTexDef->textureType = Ogre::TEX_TYPE_2D;
+    particleDepthTexDef->width = 0;
+    particleDepthTexDef->height = 0;
+    particleDepthTexDef->depth = 1;
+    particleDepthTexDef->numMipmaps = 0;
+    particleDepthTexDef->widthFactor = 0.5;
+    particleDepthTexDef->heightFactor = 0.5;
+    particleDepthTexDef->formatList = {Ogre::PF_D32_FLOAT};
+    particleDepthTexDef->fsaa = 0;
+    particleDepthTexDef->uav = false;
+    particleDepthTexDef->automipmaps = false;
+    particleDepthTexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
+    particleDepthTexDef->depthBufferId = Ogre::DepthBuffer::POOL_DEFAULT;
+
+    Ogre::TextureDefinitionBase::TextureDefinition *particleTexDef =
+        nodeDef->addTextureDefinition("particleTexture");
+    particleTexDef->textureType = Ogre::TEX_TYPE_2D;
+    particleTexDef->width = 0;
+    particleTexDef->height = 0;
+    particleTexDef->depth = 1;
+    particleTexDef->numMipmaps = 0;
+    particleTexDef->widthFactor = 0.5;
+    particleTexDef->heightFactor = 0.5;
+    particleTexDef->formatList = {Ogre::PF_R8G8B8};
+    particleTexDef->fsaa = 0;
+    particleTexDef->uav = false;
+    particleTexDef->automipmaps = false;
+    particleTexDef->hwGammaWrite = Ogre::TextureDefinitionBase::BoolFalse;
+    particleTexDef->depthBufferId = Ogre::DepthBuffer::POOL_DEFAULT;
+    particleTexDef->depthBufferFormat = Ogre::PF_D32_FLOAT;
+    particleTexDef->preferDepthTexture = true;
+    particleTexDef->fsaaExplicitResolve = false;
+
+    nodeDef->setNumTargetPass(3);
 
     Ogre::CompositorTargetDef *colorTargetDef =
         nodeDef->addTargetPass("colorTexture");
@@ -713,7 +828,26 @@ void Ogre2GpuRays::Setup1stPass()
           static_cast<Ogre::CompositorPassSceneDef *>(
           colorTargetDef->addPass(Ogre::PASS_SCENE));
       // set camera custom visibility mask when rendering laser retro
-      passScene->mVisibilityMask = 0x01000000;
+      passScene->mVisibilityMask = 0x01000000 &
+          ~Ogre2ParticleEmitter::kParticleVisibilityFlags;
+    }
+
+    Ogre::CompositorTargetDef *particleTargetDef =
+        nodeDef->addTargetPass("particleTexture");
+    particleTargetDef->setNumPasses(2);
+    {
+      // clear pass
+      Ogre::CompositorPassClearDef *passClear =
+          static_cast<Ogre::CompositorPassClearDef *>(
+          particleTargetDef->addPass(Ogre::PASS_CLEAR));
+      passClear->mColourValue = Ogre::ColourValue::Black;
+      // scene pass
+      Ogre::CompositorPassSceneDef *passScene =
+          static_cast<Ogre::CompositorPassSceneDef *>(
+          particleTargetDef->addPass(Ogre::PASS_SCENE));
+      // set camera custom visibility mask when rendering laser retro
+      passScene->mVisibilityMask =
+          Ogre2ParticleEmitter::kParticleVisibilityFlags;
     }
 
     // rt_input target - converts depth to range
@@ -733,6 +867,8 @@ void Ogre2GpuRays::Setup1stPass()
       passQuad->mMaterialName = this->dataPtr->matFirstPass->getName();
       passQuad->addQuadTextureSource(0, "depthTexture", 0);
       passQuad->addQuadTextureSource(1, "colorTexture", 0);
+      passQuad->addQuadTextureSource(2, "particleDepthTexture", 0);
+      passQuad->addQuadTextureSource(3, "particleTexture", 0);
       passQuad->mFrustumCorners =
           Ogre::CompositorPassQuadDef::VIEW_SPACE_CORNERS;
     }
@@ -796,9 +932,6 @@ void Ogre2GpuRays::Setup1stPass()
         ogreCompMgr->addWorkspace(this->scene->OgreSceneManager(),
         rt, this->dataPtr->cubeCam[i], wsDefName, false);
 
-    // add laser retro material switcher to render target listener
-    // so we can switch to use laser retro material when the camera is being
-    // updated
     Ogre::CompositorNode *node =
         this->dataPtr->ogreCompositorWorkspace1st[i]->getNodeSequence()[0];
     auto channelsTex = node->getLocalTextures();
@@ -807,10 +940,20 @@ void Ogre2GpuRays::Setup1stPass()
     {
       if (c.textures[0]->getSrcFormat() == Ogre::PF_R8G8B8)
       {
+        // add laser retro material switcher to render target listener
+        // so we can switch to use laser retro material when the camera is being
+        // updated
         this->dataPtr->laserRetroMaterialSwitcher[i].reset(
             new Ogre2LaserRetroMaterialSwitcher(this->scene));
         c.target->addListener(
             this->dataPtr->laserRetroMaterialSwitcher[i].get());
+
+        // add particle noise / scatter effects listener so we can set the
+        // amount of noise based on size of emitter
+        this->dataPtr->particleNoiseListener[i].reset(
+            new Ogre2ParticleNoiseListener(this->scene,
+            this->dataPtr->cubeCam[i], this->dataPtr->matFirstPass));
+        c.target->addListener(this->dataPtr->particleNoiseListener[i].get());
         break;
       }
     }
@@ -1058,7 +1201,7 @@ void Ogre2GpuRays::SetRangeCount(
 }
 
 //////////////////////////////////////////////////
-common::ConnectionPtr Ogre2GpuRays::ConnectNewGpuRaysFrame(
+ignition::common::ConnectionPtr Ogre2GpuRays::ConnectNewGpuRaysFrame(
     std::function<void(const float *_frame, unsigned int _width,
     unsigned int _height, unsigned int _channels,
     const std::string &/*_format*/)> _subscriber)
