@@ -17,31 +17,41 @@
 
 // Note this include is placed in the src file because
 // otherwise ogre produces compile errors
-#if defined(_MSC_VER)
-#pragma warning(push)
-#pragma warning(disable: 4005)  // Macro redefinition
-#pragma warning(disable: 5033)  // 'register' is no longer supported
+#ifdef _MSC_VER
+#pragma warning(push, 0)
 #endif
+#include <Hlms/Pbs/OgreHlmsPbs.h>
 #include <Hlms/Pbs/OgreHlmsPbsDatablock.h>
-#if defined(_MSC_VER)
+#include <Hlms/Unlit/OgreHlmsUnlit.h>
+#include <Hlms/Unlit/OgreHlmsUnlitDatablock.h>
+#include <OgreHlmsManager.h>
+#include <OgreMaterialManager.h>
+#ifdef _MSC_VER
 #pragma warning(pop)
 #endif
 
-#include <gz/common/Console.hh>
-#include <gz/common/Filesystem.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Filesystem.hh>
 
-#include "gz/rendering/ShaderParams.hh"
-#include "gz/rendering/ShaderType.hh"
-#include "gz/rendering/ogre2/Ogre2Material.hh"
-#include "gz/rendering/ogre2/Ogre2Conversions.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/ShaderParams.hh"
+#include "ignition/rendering/ShaderType.hh"
+#include "ignition/rendering/ogre2/Ogre2Material.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
 
-using namespace gz;
+
+/// \brief Private data for the Ogre2Material class
+class ignition::rendering::Ogre2MaterialPrivate
+{
+};
+
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
 Ogre2Material::Ogre2Material()
+  : dataPtr(std::make_unique<Ogre2MaterialPrivate>())
 {
 }
 
@@ -63,6 +73,13 @@ void Ogre2Material::Destroy()
   this->ogreHlmsPbs->destroyDatablock(this->ogreDatablockId);
   this->ogreDatablock = nullptr;
 
+  if (this->ogreUnlitDatablock)
+  {
+    this->ogreUnlitDatablock->getCreator()->destroyDatablock(
+        this->ogreUnlitDatablock->getName());
+    this->ogreUnlitDatablock = nullptr;
+  }
+
   // remove low level ogre material used by render targets
   if (this->ogreMaterial)
   {
@@ -132,7 +149,7 @@ void Ogre2Material::UpdateTransparency()
   Ogre::HlmsPbsDatablock::TransparencyModes mode;
   double opacity = (1.0 - this->transparency) * this->diffuse.A();
   if (math::equal(opacity, 1.0))
-     mode = Ogre::HlmsPbsDatablock::None;
+    mode = Ogre::HlmsPbsDatablock::None;
   else
     mode = Ogre::HlmsPbsDatablock::Transparent;
 
@@ -140,6 +157,42 @@ void Ogre2Material::UpdateTransparency()
   this->ogreDatablock->setTransparency(opacity, mode);
 }
 
+//////////////////////////////////////////////////
+void Ogre2Material::SetAlphaFromTexture(bool _enabled,
+    double _alpha, bool _twoSided)
+{
+  BaseMaterial::SetAlphaFromTexture(_enabled, _alpha, _twoSided);
+  if (_enabled)
+  {
+    this->ogreDatablock->setAlphaTest(Ogre::CMPF_GREATER_EQUAL);
+    Ogre::HlmsBlendblock block;
+    block.setBlendType(Ogre::SBT_TRANSPARENT_ALPHA);
+    this->ogreDatablock->setBlendblock(block);
+  }
+  else
+  {
+    this->ogreDatablock->setAlphaTest(Ogre::CMPF_ALWAYS_PASS);
+  }
+  this->ogreDatablock->setAlphaTestThreshold(_alpha);
+  this->ogreDatablock->setTwoSidedLighting(_twoSided);
+}
+
+//////////////////////////////////////////////////
+float Ogre2Material::RenderOrder() const
+{
+  return this->renderOrder;
+}
+
+//////////////////////////////////////////////////
+void Ogre2Material::SetRenderOrder(const float _renderOrder)
+{
+  this->renderOrder = _renderOrder;
+  Ogre::HlmsMacroblock macroblock(
+      *this->ogreDatablock->getMacroblock());
+  macroblock.mDepthBiasConstant = _renderOrder;
+  this->ogreDatablock->setMacroblock(macroblock);
+}
+
 //////////////////////////////////////////////////
 bool Ogre2Material::ReceiveShadows() const
 {
@@ -344,6 +397,56 @@ void Ogre2Material::ClearEmissiveMap()
   this->ogreDatablock->setTexture(Ogre::PBSM_EMISSIVE, 0, Ogre::TexturePtr());
 }
 
+//////////////////////////////////////////////////
+bool Ogre2Material::HasLightMap() const
+{
+  return !this->lightMapName.empty();
+}
+
+//////////////////////////////////////////////////
+std::string Ogre2Material::LightMap() const
+{
+  return this->lightMapName;
+}
+
+//////////////////////////////////////////////////
+unsigned int Ogre2Material::LightMapTexCoordSet() const
+{
+  return this->lightMapUvSet;
+}
+
+//////////////////////////////////////////////////
+void Ogre2Material::SetLightMap(const std::string &_name, unsigned int _uvSet)
+{
+  if (_name.empty())
+  {
+    this->ClearLightMap();
+    return;
+  }
+
+  this->lightMapName = _name;
+  this->lightMapUvSet = _uvSet;
+
+  // reserve detail map 0 for light map
+  Ogre::PbsTextureTypes type = Ogre::PBSM_DETAIL0;
+
+  // lightmap usually uses a different tex coord set
+  this->SetTextureMapImpl(this->lightMapName, type);
+
+  this->ogreDatablock->setTextureUvSource(type, this->lightMapUvSet);
+
+  // PBSM_BLEND_OVERLAY and PBSM_BLEND_MULTIPLY2X produces better results
+  this->ogreDatablock->setDetailMapBlendMode(0, Ogre::PBSM_BLEND_OVERLAY);
+}
+
+//////////////////////////////////////////////////
+void Ogre2Material::ClearLightMap()
+{
+  this->lightMapName = "";
+  this->lightMapUvSet = 0u;
+  this->ogreDatablock->setTexture(Ogre::PBSM_DETAIL0, 0, Ogre::TexturePtr());
+}
+
 //////////////////////////////////////////////////
 void Ogre2Material::SetRoughness(const float _roughness)
 {
@@ -443,6 +546,17 @@ void Ogre2Material::SetTextureMapImpl(const std::string &_texture,
 
   this->ogreDatablock->setTexture(_type, texLocation.xIdx, texLocation.texture,
       &samplerBlockRef);
+
+  // disable alpha from texture if texture does not have an alpha channel
+  // otherwise this becomes a transparent material
+  if (_type == Ogre::PBSM_DIFFUSE)
+  {
+    if (this->TextureAlphaEnabled() && !texLocation.texture->hasAlpha())
+    {
+      this->SetAlphaFromTexture(false, this->AlphaThreshold(),
+          this->TwoSidedEnabled());
+    }
+  }
 }
 
 //////////////////////////////////////////////////
@@ -522,3 +636,59 @@ void Ogre2Material::SetDepthWriteEnabled(bool _enabled)
   macroblock.mDepthWrite = _enabled;
   this->ogreDatablock->setMacroblock(macroblock);
 }
+
+//////////////////////////////////////////////////
+Ogre::HlmsUnlitDatablock *Ogre2Material::UnlitDatablock()
+{
+  if (!this->ogreUnlitDatablock)
+  {
+    // Create the internal UNLIT material.
+    Ogre::Root *root = Ogre2RenderEngine::Instance()->OgreRoot();
+    Ogre::HlmsManager *hlmsManager = root->getHlmsManager();
+    Ogre::HlmsUnlit *ogreHlmsUnlit = static_cast<Ogre::HlmsUnlit *>(
+        hlmsManager->getHlms(Ogre::HLMS_UNLIT));
+    if (!ogreHlmsUnlit)
+    {
+      ignerr << "Ogre HLMS UNLIT not ready. Is Ogre2 Render Engine "
+             << "initiallized?" << std::endl;
+      return nullptr;
+    }
+
+    std::string unlitName = this->name + "::Unlit";
+    this->ogreUnlitDatablock =
+        static_cast<Ogre::HlmsUnlitDatablock *>(
+        ogreHlmsUnlit->createDatablock(
+        unlitName, unlitName,
+        Ogre::HlmsMacroblock(), Ogre::HlmsBlendblock(), Ogre::HlmsParamVec()));
+  }
+
+  this->FillUnlitDatablock(this->ogreUnlitDatablock);
+  return this->ogreUnlitDatablock;
+}
+
+//////////////////////////////////////////////////
+void Ogre2Material::FillUnlitDatablock(Ogre::HlmsUnlitDatablock *_datablock)
+    const
+{
+  if (!this->textureName.empty())
+  {
+    std::string baseName = common::basename(this->textureName);
+    Ogre::HlmsTextureManager *hlmsTextureManager =
+        this->ogreHlmsPbs->getHlmsManager()->getTextureManager();
+    Ogre::HlmsTextureManager::TextureLocation texLocation =
+        hlmsTextureManager->createOrRetrieveTexture(baseName,
+        this->ogreDatablock->suggestMapTypeBasedOnTextureType(
+        Ogre::PBSM_DIFFUSE));
+    _datablock->setTexture(0, texLocation.xIdx, texLocation.texture);
+  }
+
+  auto samplerblock = this->ogreDatablock->getSamplerblock(Ogre::PBSM_DIFFUSE);
+  if (samplerblock)
+    _datablock->setSamplerblock(0, *samplerblock);
+  _datablock->setMacroblock(this->ogreDatablock->getMacroblock());
+  _datablock->setBlendblock(this->ogreDatablock->getBlendblock());
+
+  _datablock->setUseColour(true);
+  Ogre::Vector3 c = this->ogreDatablock->getDiffuse();
+  _datablock->setColour(Ogre::ColourValue(c.x, c.y, c.z));
+}
