@@ -15,20 +15,31 @@
  *
 */
 
-#include "gz/common/Console.hh"
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2MaterialSwitcher.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
-#include "gz/rendering/RenderTypes.hh"
+#include "ignition/common/Console.hh"
+#include "ignition/rendering/ogre2/Ogre2MaterialSwitcher.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/RenderTypes.hh"
 
-using namespace gz;
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <OgreItem.h>
+#include <OgreMaterialManager.h>
+#include <OgrePass.h>
+#include <OgreSceneManager.h>
+#include <OgreTechnique.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
+
+using namespace ignition;
 using namespace rendering;
 
 
 /////////////////////////////////////////////////
 Ogre2MaterialSwitcher::Ogre2MaterialSwitcher(Ogre2ScenePtr _scene)
 {
-  this->currentColor = math::Color(0.0f, 0.0f, 0.1f);
+  this->currentColor = ignition::math::Color(0.0, 0.0, 0.1);
   this->scene = _scene;
 
   // plain opaque material
@@ -127,7 +138,7 @@ void Ogre2MaterialSwitcher::postRenderTargetUpdate(
 
 /////////////////////////////////////////////////
 std::string Ogre2MaterialSwitcher::EntityName(
-    const math::Color &_color) const
+    const ignition::math::Color &_color) const
 {
   auto iter = this->colorDict.find(_color.AsRGBA());
 
@@ -148,7 +159,7 @@ void Ogre2MaterialSwitcher::NextColor()
 /////////////////////////////////////////////////
 void Ogre2MaterialSwitcher::Reset()
 {
-  this->currentColor = math::Color(
+  this->currentColor = ignition::math::Color(
       0.0, 0.0, 0.0);
   this->colorDict.clear();
 }
