@@ -18,28 +18,59 @@
 
 #include <sstream>
 
-#include <gz/common/Console.hh>
-#include <gz/common/Material.hh>
-#include <gz/common/MeshManager.hh>
-#include <gz/common/Skeleton.hh>
-#include <gz/common/SubMesh.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Material.hh>
+#include <ignition/common/MeshManager.hh>
+#include <ignition/common/Skeleton.hh>
+#include <ignition/common/SkeletonAnimation.hh>
+#include <ignition/common/SubMesh.hh>
 
-#include <gz/math/Matrix4.hh>
+#include <ignition/math/Matrix4.hh>
 
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2Mesh.hh"
-#include "gz/rendering/ogre2/Ogre2MeshFactory.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
-#include "gz/rendering/ogre2/Ogre2Storage.hh"
+#include "ignition/rendering/ogre2/Ogre2Conversions.hh"
+#include "ignition/rendering/ogre2/Ogre2Mesh.hh"
+#include "ignition/rendering/ogre2/Ogre2MeshFactory.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/ogre2/Ogre2Storage.hh"
 
-using namespace gz;
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
+#include <OgreHardwareBufferManager.h>
+#include <OgreItem.h>
+#include <OgreKeyFrame.h>
+#include <OgreMesh2.h>
+#include <OgreMeshManager.h>
+#include <OgreMeshManager2.h>
+#include <OgreOldBone.h>
+#include <OgreOldSkeletonManager.h>
+#include <OgreSceneManager.h>
+#include <OgreSkeleton.h>
+#include <OgreSubItem.h>
+#include <OgreSubMesh.h>
+#include <OgreSubMesh2.h>
+#ifdef _MSC_VER
+  #pragma warning(pop)
+#endif
+
+/// \brief Private data for the Ogre2MeshFactory class
+class ignition::rendering::Ogre2MeshFactoryPrivate
+{
+};
+
+/// \brief Private data for the Ogre2SubMeshStoreFactory class
+class ignition::rendering::Ogre2SubMeshStoreFactoryPrivate
+{
+};
+
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
 Ogre2MeshFactory::Ogre2MeshFactory(Ogre2ScenePtr _scene) :
-  scene(_scene)
+  scene(_scene), dataPtr(std::make_unique<Ogre2MeshFactoryPrivate>())
 {
 }
 
@@ -151,8 +182,8 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
     group = Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME;
     ogreMesh = Ogre::v1::MeshManager::getSingleton().createManual(name, group);
 
+    // load skeleton
     Ogre::v1::SkeletonPtr ogreSkeleton;
-
     if (_desc.mesh->HasSkeleton())
     {
       common::SkeletonPtr skel = _desc.mesh->MeshSkeleton();
@@ -161,6 +192,7 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
         Ogre::ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME,
         true);
 
+      // load bones
       for (unsigned int i = 0; i < skel->NodeCount(); i++)
       {
         common::SkeletonNode *node = skel->NodeByHandle(i);
@@ -178,6 +210,58 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
         bone->setManuallyControlled(true);
         bone->setInitialState();
       }
+
+      // load skeletal animations
+      for (unsigned int i = 0; i < skel->AnimationCount(); ++i)
+      {
+        common::SkeletonAnimation *skelAnim = skel->Animation(i);
+        if (ogreSkeleton->hasAnimation(skelAnim->Name()))
+        {
+          continue;
+        }
+
+        Ogre::v1::Animation *ogreAnim = ogreSkeleton->createAnimation(
+            skelAnim->Name(), skelAnim->Length());
+
+        for (unsigned int j = 0; j < skel->NodeCount(); ++j)
+        {
+          common::SkeletonNode *node = skel->NodeByHandle(j);
+          common::NodeAnimation *nodeAnim = skelAnim->NodeAnimationByName(
+              node->Name());
+          if (!nodeAnim)
+          {
+            continue;
+          }
+
+          Ogre::v1::OldBone *bone = ogreSkeleton->getBone(node->Name());
+          Ogre::v1::OldNodeAnimationTrack *ogreNodeAnimTrack =
+              ogreAnim->createOldNodeTrack(j, bone);
+
+          // set up transform (needed for bvh)
+          math::Matrix4d alignTrans = skel->AlignTranslation(i, node->Name());
+          math::Matrix4d alignRot = skel->AlignRotation(i, node->Name());
+
+          for (unsigned int k = 0; k < nodeAnim->FrameCount(); ++k)
+          {
+            std::pair<double, math::Matrix4d> keyFrame = nodeAnim->KeyFrame(k);
+            Ogre::v1::TransformKeyFrame *kf =
+                ogreNodeAnimTrack->createNodeKeyFrame(keyFrame.first);
+
+            math::Matrix4d p = keyFrame.second;
+
+            // apply anim-skin transform
+            p = alignTrans * p * alignRot;
+
+            auto rot = bone->getOrientation().Inverse() *
+                Ogre2Conversions::Convert(p.Rotation());
+            auto pos = Ogre2Conversions::Convert(p.Translation()) -
+                bone->getPosition();
+            kf->setRotation(rot);
+            kf->setTranslate(pos);
+          }
+        }
+      }
+
       ogreMesh->setSkeletonName(_desc.mesh->Name() + "_skeleton");
     }
 
@@ -268,11 +352,15 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
       // TODO(anyone): specular colors
 
       // two dimensional texture coordinates
-      if (subMesh.TexCoordCount() > 0)
+      // add all texture coordinate sets
+      for (unsigned int k = 0u; k < subMesh.TexCoordSetCount(); ++k)
       {
-        vertexDecl->addElement(0, currOffset, Ogre::VET_FLOAT2,
-            Ogre::VES_TEXTURE_COORDINATES, 0);
-        currOffset += Ogre::v1::VertexElement::getTypeSize(Ogre::VET_FLOAT2);
+        if (subMesh.TexCoordCountBySet(k) > 0)
+        {
+          vertexDecl->addElement(0, currOffset, Ogre::VET_FLOAT2,
+              Ogre::VES_TEXTURE_COORDINATES, k);
+          currOffset += Ogre::v1::VertexElement::getTypeSize(Ogre::VET_FLOAT2);
+        }
       }
 
       // allocate the vertex buffer
@@ -310,6 +398,7 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
         *vertices++ = subMesh.Vertex(j).Y();
         *vertices++ = subMesh.Vertex(j).Z();
 
+        // Add all normals
         if (subMesh.NormalCount() > 0)
         {
           *vertices++ = subMesh.Normal(j).X();
@@ -317,10 +406,14 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
           *vertices++ = subMesh.Normal(j).Z();
         }
 
-        if (subMesh.TexCoordCount() > 0)
+        // Add all texture coordinate sets
+        for (unsigned int k = 0u; k < subMesh.TexCoordSetCount(); ++k)
         {
-          *vertices++ = subMesh.TexCoord(j).X();
-          *vertices++ = subMesh.TexCoord(j).Y();
+          if (subMesh.TexCoordCountBySet(k) > 0u)
+          {
+            *vertices++ = subMesh.TexCoordBySet(j, k).X();
+            *vertices++ = subMesh.TexCoordBySet(j, k).Y();
+          }
         }
       }
 
@@ -404,6 +497,15 @@ bool Ogre2MeshFactory::LoadImpl(const MeshDescriptor &_desc)
     return false;
   }
 
+  if (ogreMesh->getNumSubMeshes() == 0u)
+  {
+    std::string msg = "Unable to load mesh: '" + _desc.meshName + "'";
+    if (!_desc.subMeshName.empty())
+      msg += ", submesh: '" + _desc.subMeshName + "'";
+    msg += ". Mesh will be empty.";
+    ignwarn << msg << std::endl;
+  }
+
   return true;
 }
 
@@ -445,7 +547,8 @@ bool Ogre2MeshFactory::Validate(const MeshDescriptor &_desc)
 Ogre2SubMeshStoreFactory::Ogre2SubMeshStoreFactory(Ogre2ScenePtr _scene,
     Ogre::Item *_item) :
   scene(_scene),
-  ogreItem(_item)
+  ogreItem(_item),
+  dataPtr(new Ogre2SubMeshStoreFactoryPrivate)
 {
   this->CreateNameList();
 }
