@@ -20,6 +20,7 @@
 # include <X11/Xlib.h>
 # include <X11/Xutil.h>
 # include <GL/glx.h>
+# include <GL/glxext.h>
 #endif
 
 #ifdef _WIN32
@@ -27,31 +28,31 @@
   // pulled in by anybody (e.g., Boost).
   #include <Winsock2.h>
 #endif
-#include <gz/common/Console.hh>
-#include <gz/common/Filesystem.hh>
-#include <gz/common/Util.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/Filesystem.hh>
+#include <ignition/common/Util.hh>
 
-#include <gz/plugin/Register.hh>
+#include <ignition/plugin/Register.hh>
 
-#include "gz/rendering/RenderEngineManager.hh"
-#include "gz/rendering/ogre2/Ogre2Includes.hh"
-#include "gz/rendering/ogre2/Ogre2RenderEngine.hh"
-#include "gz/rendering/ogre2/Ogre2RenderTypes.hh"
-#include "gz/rendering/ogre2/Ogre2Scene.hh"
-#include "gz/rendering/ogre2/Ogre2Storage.hh"
+#include "ignition/rendering/RenderEngineManager.hh"
+#include "ignition/rendering/ogre2/Ogre2Includes.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderEngine.hh"
+#include "ignition/rendering/ogre2/Ogre2RenderTypes.hh"
+#include "ignition/rendering/ogre2/Ogre2Scene.hh"
+#include "ignition/rendering/ogre2/Ogre2Storage.hh"
 
 
-class gz::rendering::Ogre2RenderEnginePrivate
+class ignition::rendering::Ogre2RenderEnginePrivate
 {
 #if !defined(__APPLE__) && !defined(_WIN32)
-  public: XVisualInfo *dummyVisual = nullptr;
+  public: GLXFBConfig* dummyFBConfigs = nullptr;
 #endif
 
   /// \brief A list of supported fsaa levels
   public: std::vector<unsigned int> fsaaLevels;
 };
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 //////////////////////////////////////////////////
@@ -82,11 +83,9 @@ Ogre2RenderEngine::Ogre2RenderEngine() :
   std::string ogrePath = std::string(OGRE2_RESOURCE_PATH);
   this->ogrePaths.push_back(ogrePath);
 
-#ifdef __APPLE__
-  // on OSX the plugins may be placed in the parent lib directory
-  if (ogrePath.rfind("OGRE") == ogrePath.size()-4u)
-    this->ogrePaths.push_back(ogrePath.substr(0, ogrePath.size()-5));
-#endif
+  const char *env = std::getenv("OGRE2_RESOURCE_PATH");
+  if (env)
+    this->ogrePaths.push_back(std::string(env));
 }
 
 //////////////////////////////////////////////////
@@ -123,7 +122,7 @@ void Ogre2RenderEngine::Destroy()
   delete this->ogreLogManager;
   this->ogreLogManager = nullptr;
 
-#if !defined(__APPLE__) && !defined(_WIN32)
+#if not (__APPLE__ || _WIN32)
   if (this->dummyDisplay)
   {
     Display *x11Display = static_cast<Display*>(this->dummyDisplay);
@@ -132,8 +131,8 @@ void Ogre2RenderEngine::Destroy()
     XDestroyWindow(x11Display, this->dummyWindowId);
     XCloseDisplay(x11Display);
     this->dummyDisplay = nullptr;
-    XFree(this->dataPtr->dummyVisual);
-    this->dataPtr->dummyVisual = nullptr;
+    XFree(this->dataPtr->dummyFBConfigs);
+    this->dataPtr->dummyFBConfigs = nullptr;
   }
 #endif
 }
@@ -141,7 +140,7 @@ void Ogre2RenderEngine::Destroy()
 //////////////////////////////////////////////////
 bool Ogre2RenderEngine::IsEnabled() const
 {
-  return BaseRenderEngine::IsEnabled();
+  return this->initialized;
 }
 
 //////////////////////////////////////////////////
@@ -315,7 +314,7 @@ void Ogre2RenderEngine::CreateLogger()
 {
   // create log file path
   std::string logPath;
-  common::env(IGN_HOMEDIR, logPath);
+  ignition::common::env(IGN_HOMEDIR, logPath);
   logPath = common::joinPaths(logPath, ".ignition", "rendering");
   common::createDirectories(logPath);
   logPath = common::joinPaths(logPath, "ogre2.log");
@@ -328,7 +327,7 @@ void Ogre2RenderEngine::CreateLogger()
 //////////////////////////////////////////////////
 void Ogre2RenderEngine::CreateContext()
 {
-#if !defined(__APPLE__) && !defined(_WIN32)
+#if not (__APPLE__ || _WIN32)
   // create X11 display
   this->dummyDisplay = XOpenDisplay(0);
   Display *x11Display = static_cast<Display*>(this->dummyDisplay);
@@ -342,15 +341,22 @@ void Ogre2RenderEngine::CreateContext()
   // create X11 visual
   int screenId = DefaultScreen(x11Display);
 
-  int attributeList[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 16,
-      GLX_STENCIL_SIZE, 8, None };
+  int attributeList[] = {
+    GLX_RENDER_TYPE, GLX_RGBA_BIT,
+    GLX_DOUBLEBUFFER, True,
+    GLX_DEPTH_SIZE, 16,
+    GLX_STENCIL_SIZE, 8,
+    None
+  };
 
-  this->dataPtr->dummyVisual =
-      glXChooseVisual(x11Display, screenId, attributeList);
+  int nelements = 0;
 
-  if (!this->dataPtr->dummyVisual)
+  this->dataPtr->dummyFBConfigs =
+      glXChooseFBConfig(x11Display, screenId, attributeList, &nelements);
+
+  if (nelements <= 0)
   {
-    ignerr << "Unable to create glx visual" << std::endl;
+    ignerr << "Unable to create glx fbconfig" << std::endl;
     return;
   }
 
@@ -358,8 +364,30 @@ void Ogre2RenderEngine::CreateContext()
   this->dummyWindowId = XCreateSimpleWindow(x11Display,
       RootWindow(this->dummyDisplay, screenId), 0, 0, 1, 1, 0, 0, 0);
 
-  this->dummyContext = glXCreateContext(x11Display, this->dataPtr->dummyVisual,
-                                        nullptr, 1);
+  PFNGLXCREATECONTEXTATTRIBSARBPROC glXCreateContextAttribsARB = 0;
+  glXCreateContextAttribsARB =
+      (PFNGLXCREATECONTEXTATTRIBSARBPROC)
+      glXGetProcAddress((const GLubyte*)"glXCreateContextAttribsARB");
+
+  if (glXCreateContextAttribsARB)
+  {
+    int contextAttribs[] = {
+      GLX_CONTEXT_MAJOR_VERSION_ARB, 3,
+      GLX_CONTEXT_MINOR_VERSION_ARB, 3,
+      None
+    };
+    this->dummyContext =
+        glXCreateContextAttribsARB(x11Display,
+                                  this->dataPtr->dummyFBConfigs[0], nullptr,
+                                  1, contextAttribs);
+  }
+  else
+  {
+    ignwarn << "glXCreateContextAttribsARB() not found" << std::endl;
+    this->dummyContext = glXCreateNewContext(x11Display,
+                                             this->dataPtr->dummyFBConfigs[0],
+                                             GLX_RGBA_TYPE, nullptr, 1);
+  }
 
   GLXContext x11Context = static_cast<GLXContext>(this->dummyContext);
 
@@ -560,6 +588,8 @@ void Ogre2RenderEngine::CreateResources()
         std::make_pair(p + "/materials/programs", "General"));
     archNames.push_back(
         std::make_pair(p + "/materials/scripts", "General"));
+    archNames.push_back(
+        std::make_pair(p + "/materials/textures", "General"));
 
     for (auto aiter = archNames.begin(); aiter != archNames.end(); ++aiter)
     {
@@ -785,5 +815,5 @@ Ogre::v1::OverlaySystem *Ogre2RenderEngine::OverlaySystem() const
 }
 
 // Register this plugin
-IGNITION_ADD_PLUGIN(Ogre2RenderEnginePlugin,
-                    rendering::RenderEnginePlugin)
+IGNITION_ADD_PLUGIN(ignition::rendering::Ogre2RenderEnginePlugin,
+                    ignition::rendering::RenderEnginePlugin)
