@@ -17,15 +17,15 @@
 
 #include <gtest/gtest.h>
 
-#include <gz/common/Console.hh>
+#include <ignition/common/Console.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
-#include "gz/rendering/Light.hh"
-#include "gz/rendering/RenderEngine.hh"
-#include "gz/rendering/RenderingIface.hh"
-#include "gz/rendering/Scene.hh"
+#include "ignition/rendering/Light.hh"
+#include "ignition/rendering/RenderEngine.hh"
+#include "ignition/rendering/RenderingIface.hh"
+#include "ignition/rendering/Scene.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 class LightTest : public testing::Test,
@@ -74,6 +74,10 @@ void LightTest::Light(const std::string &_renderEngine)
   light->SetCastShadows(false);
   EXPECT_FALSE(light->CastShadows());
 
+  // intensity
+  light->SetIntensity(1.25);
+  EXPECT_NEAR(1.25, light->Intensity(), 1e-6);
+
   // attenuation
   // Checking near because Ogre stores it as float
   light->SetAttenuationConstant(0.6);
@@ -145,7 +149,7 @@ void LightTest::Light(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -156,7 +160,7 @@ TEST_P(LightTest, Light)
 
 INSTANTIATE_TEST_CASE_P(Light, LightTest,
     RENDER_ENGINE_VALUES,
-    PrintToStringParam());
+    ignition::rendering::PrintToStringParam());
 
 int main(int argc, char **argv)
 {
