@@ -18,19 +18,19 @@
 #include <gtest/gtest.h>
 #include <string>
 
-#include <gz/common/Console.hh>
-#include <gz/common/MeshManager.hh>
-#include <gz/common/Skeleton.hh>
-#include <gz/common/SkeletonAnimation.hh>
+#include <ignition/common/Console.hh>
+#include <ignition/common/MeshManager.hh>
+#include <ignition/common/Skeleton.hh>
+#include <ignition/common/SkeletonAnimation.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
-#include "gz/rendering/Camera.hh"
-#include "gz/rendering/Mesh.hh"
-#include "gz/rendering/RenderEngine.hh"
-#include "gz/rendering/RenderingIface.hh"
-#include "gz/rendering/Scene.hh"
+#include "ignition/rendering/Camera.hh"
+#include "ignition/rendering/Mesh.hh"
+#include "ignition/rendering/RenderEngine.hh"
+#include "ignition/rendering/RenderingIface.hh"
+#include "ignition/rendering/Scene.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 class MeshTest : public testing::Test,
@@ -39,12 +39,12 @@ class MeshTest : public testing::Test,
   /// \brief Test mesh and submesh basic API
   public: void MeshSubMesh(const std::string &_renderEngine);
 
-  /// \brief Test mesh skeleton
-  public: void MeshSkeleton(const std::string &_renderEngine);
+  /// \brief Test mesh skeleton animation API
+  public: void MeshSkeletonAnimation(const std::string &_renderEngine);
 
   public: const std::string TEST_MEDIA_PATH =
         common::joinPaths(std::string(PROJECT_SOURCE_PATH),
-        "test", "media", "skeleton");
+        "test", "media", "meshes");
 };
 
 /////////////////////////////////////////////////
@@ -77,13 +77,6 @@ void MeshTest::MeshSubMesh(const std::string &_renderEngine)
 
   EXPECT_EQ(submesh, mesh->SubMeshByName(submesh->Name()));
 
-  EXPECT_FALSE(mesh->HasSkeleton());
-
-  std::map<std::string, ignition::math::Matrix4d> skinFrames;
-  mesh->SetSkeletonLocalTransforms(skinFrames);
-  auto skeletonLocalTransforms = mesh->SkeletonLocalTransforms();
-  EXPECT_EQ(0u, skeletonLocalTransforms.size());
-
   // test submesh API
   MaterialPtr mat = submesh->Material();
   ASSERT_TRUE(mat != nullptr);
@@ -95,8 +88,6 @@ void MeshTest::MeshSubMesh(const std::string &_renderEngine)
   EXPECT_EQ(matClone, submesh->Material());
   EXPECT_NE(mat, submesh->Material());
 
-  submesh->SetMaterial(MaterialPtr(), false);
-
   submesh->SetMaterial("Default/White", false);
   EXPECT_EQ("Default/White", submesh->Material()->Name());
 
@@ -106,11 +97,17 @@ void MeshTest::MeshSubMesh(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
-void MeshTest::MeshSkeleton(const std::string &_renderEngine)
+TEST_P(MeshTest, MeshSubMesh)
+{
+  MeshSubMesh(GetParam());
+}
+
+/////////////////////////////////////////////////
+void MeshTest::MeshSkeletonAnimation(const std::string &_renderEngine)
 {
   RenderEngine *engine = rendering::engine(_renderEngine);
   if (!engine)
@@ -121,96 +118,93 @@ void MeshTest::MeshSkeleton(const std::string &_renderEngine)
   }
 
   ScenePtr scene = engine->CreateScene("scene");
-  ASSERT_NE(nullptr, scene);
+  ASSERT_TRUE(scene != nullptr);
 
-  VisualPtr root = scene->RootVisual();
+  // test box mesh with no skeleton animation
+  MeshDescriptor boxDescriptor("unit_box");
+  MeshPtr boxMesh = scene->CreateMesh(boxDescriptor);
+  ASSERT_TRUE(boxMesh != nullptr);
 
-  // create a visual for the actor, attach mesh and get skeleton
-  // Skeleton will be animated by GlutWindow
-  VisualPtr actorVisual = scene->CreateVisual("actor");
-  actorVisual->SetLocalPosition(0, 0, 0);
-  actorVisual->SetLocalRotation(0, 0, 0);
+  EXPECT_FALSE(boxMesh->HasSkeleton());
+  EXPECT_NO_THROW(boxMesh->SetSkeletonAnimationEnabled("invalid", false));
+  EXPECT_FALSE(boxMesh->SkeletonAnimationEnabled("invalid"));
+  EXPECT_TRUE(boxMesh->SkeletonLocalTransforms().empty());
 
+  // test mesh with skeleton animation
   MeshDescriptor descriptor;
   descriptor.meshName = common::joinPaths(TEST_MEDIA_PATH, "walk.dae");
   common::MeshManager *meshManager = common::MeshManager::Instance();
   descriptor.mesh = meshManager->Load(descriptor.meshName);
-
   MeshPtr mesh = scene->CreateMesh(descriptor);
-  actorVisual->AddGeometry(mesh);
-  root->AddChild(actorVisual);
-
-  common::SkeletonPtr skel;
-
-  if (mesh && descriptor.mesh->HasSkeleton())
-  {
-    skel = descriptor.mesh->MeshSkeleton();
-
-    if (!skel || skel->AnimationCount() == 0)
-    {
-      FAIL();
-    }
-  }
-  else
-  {
-    FAIL();
-  }
 
   EXPECT_TRUE(mesh->HasSkeleton());
+  EXPECT_FALSE(mesh->SkeletonLocalTransforms().empty());
 
-  std::string bvhFile = common::joinPaths(TEST_MEDIA_PATH, "cmu-13_26.bvh");
+  auto skel = descriptor.mesh->MeshSkeleton();
+  ASSERT_EQ(1u, skel->AnimationCount());
 
-  double scale = 0.055;
-  skel->AddBvhAnimation(bvhFile, scale);
+  std::string animName = skel->Animation(0u)->Name();
+  EXPECT_FALSE(mesh->SkeletonAnimationEnabled(animName));
 
-  int g_animIdx = 1;
-  auto * skelAnim = skel->Animation(g_animIdx);
-  for (double i = 0; i < 10; i+=0.01)
+  mesh->SetSkeletonAnimationEnabled(animName, true);
+  EXPECT_TRUE(mesh->SkeletonAnimationEnabled(animName));
+
+  EXPECT_NO_THROW(mesh->UpdateSkeletonAnimation(
+      std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+      std::chrono::duration<double>(0.0))));
+  EXPECT_NO_THROW(mesh->UpdateSkeletonAnimation(
+      std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+      std::chrono::duration<double>(1.0))));
+  EXPECT_NO_THROW(mesh->UpdateSkeletonAnimation(
+      std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+      std::chrono::duration<double>(-11.0))));
+  EXPECT_NO_THROW(mesh->UpdateSkeletonAnimation(
+      std::chrono::duration_cast<std::chrono::steady_clock::duration>(
+      std::chrono::duration<double>(1234.5))));
+
+  // verify default skeleton bone weight
+  std::unordered_map<std::string, float> weights = mesh->SkeletonWeights();
+  EXPECT_FALSE(weights.empty());
+  EXPECT_EQ(skel->NodeCount(), weights.size());
+  for (auto &it : weights)
   {
-    std::map<std::string, ignition::math::Matrix4d> animFrames;
-    animFrames = skelAnim->PoseAt(i, true);
-
-    std::map<std::string, ignition::math::Matrix4d> skinFrames;
-
-    for (auto pair : animFrames)
-    {
-      std::string animName = pair.first;
-      auto animTf = pair.second;
-
-      std::string skinName = skel->NodeNameAnimToSkin(g_animIdx, animName);
-      ignition::math::Matrix4d skinTf =
-              skel->AlignTranslation(g_animIdx, animName)
-              * animTf * skel->AlignRotation(g_animIdx, animName);
-
-      skinFrames[skinName] = skinTf;
-    }
-
-    mesh->SetSkeletonLocalTransforms(skinFrames);
+    EXPECT_NE(nullptr, skel->NodeByName(it.first));
+    EXPECT_FLOAT_EQ(1.0, it.second);
   }
 
-  auto skeletonLocalTransforms = mesh->SkeletonLocalTransforms();
-  EXPECT_EQ(31u, skeletonLocalTransforms.size());
+  // change a bone weight and verify
+  std::string nodeName  = skel->RootNode()->Name();
+  EXPECT_EQ(1u, weights.count(nodeName));
+  weights[nodeName] = 0.5;
+  mesh->SetSkeletonWeights(weights);
+
+  auto newWeights = mesh->SkeletonWeights();
+  EXPECT_FALSE(newWeights.empty());
+  EXPECT_EQ(skel->NodeCount(), newWeights.size());
+  EXPECT_FLOAT_EQ(0.5, newWeights[nodeName]);
+  for (auto &it : newWeights)
+  {
+    EXPECT_NE(nullptr, skel->NodeByName(it.first));
+    if (it.first != nodeName)
+    {
+      EXPECT_FLOAT_EQ(1.0, it.second);
+    }
+  }
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
-TEST_P(MeshTest, MeshSubMesh)
+TEST_P(MeshTest, MeshSkeletonAnimation)
 {
-  MeshSubMesh(GetParam());
-}
-
-/////////////////////////////////////////////////
-TEST_P(MeshTest, MeshSkeleton)
-{
-  MeshSkeleton(GetParam());
+  MeshSkeletonAnimation(GetParam());
 }
 
 INSTANTIATE_TEST_CASE_P(Mesh, MeshTest,
     RENDER_ENGINE_VALUES,
-    PrintToStringParam());
+    ignition::rendering::PrintToStringParam());
 
 int main(int argc, char **argv)
 {
