@@ -17,20 +17,26 @@
 
 #include <gtest/gtest.h>
 
-#include <gz/common/Console.hh>
+#include <ignition/common/Console.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
-#include "gz/rendering/RenderEngine.hh"
-#include "gz/rendering/RenderTarget.hh"
-#include "gz/rendering/RenderingIface.hh"
-#include "gz/rendering/Scene.hh"
+#include "ignition/rendering/RenderEngine.hh"
+#include "ignition/rendering/RenderTarget.hh"
+#include "ignition/rendering/RenderingIface.hh"
+#include "ignition/rendering/Scene.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 class SceneTest : public testing::Test,
                   public testing::WithParamInterface<const char *>
 {
+  // Documentation inherited
+  public: void SetUp() override
+  {
+    ignition::common::Console::SetVerbosity(4);
+  }
+
   public: void Scene(const std::string &_renderEngine);
   public: void Nodes(const std::string &_renderEngine);
 
@@ -46,9 +52,14 @@ class SceneTest : public testing::Test,
   /// \brief Test creating and destroying materials
   public: void Materials(const std::string &_renderEngine);
 
-  public: const std::string TEST_MEDIA_PATH =
-        common::joinPaths(std::string(PROJECT_SOURCE_PATH),
-        "test", "media", "skeleton");
+  /// \brief Test setting and getting Time
+  public: void Time(const std::string &_renderEngine);
+
+  /// \brief Test background material
+  public: void BackgroundMaterial(const std::string &_renderEngine);
+
+  /// \brief Test enablng sky
+  public: void Sky(const std::string &_renderEngine);
 };
 
 /////////////////////////////////////////////////
@@ -76,7 +87,7 @@ void SceneTest::Scene(const std::string &_renderEngine)
 
   // TODO(anyone) gradient background color and render window only supported
   // by ogre
-  if (_renderEngine == "ogre2")
+  if (_renderEngine == "ogre")
   {
     EXPECT_FALSE(scene->IsGradientBackgroundColor());
 
@@ -114,7 +125,7 @@ void SceneTest::Scene(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -198,7 +209,7 @@ void SceneTest::Nodes(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -300,7 +311,7 @@ void SceneTest::RemoveNodes(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -445,85 +456,11 @@ void SceneTest::DestroyNodes(const std::string &_renderEngine)
   EXPECT_FALSE(scene->HasVisual(childB));
   EXPECT_FALSE(scene->HasVisual(childAA));
 
-  auto gizmoVisual = scene->CreateGizmoVisual("gizmo_visual");
-  auto planeVisual = scene->CreatePlane();
-  auto meshVisual = scene->CreateMesh(
-    common::joinPaths(TEST_MEDIA_PATH, "walk.dae"));
-
-  common::MeshPtr mesh(new common::Mesh());
-  meshVisual = scene->CreateMesh(mesh.get());
-
-  scene->DestroyVisuals();
-
-  common::Material mat(math::Color(1.0f, 0.5f, 0.2f, 1.0f));
-  auto materialVisual = scene->CreateMaterial(mat);
-  scene->DestroyMaterial(materialVisual);
-  scene->DestroyMaterial(MaterialPtr());
-
-  auto depthCameraSensor = scene->CreateDepthCamera();
-  scene->DestroySensors();
-  depthCameraSensor = scene->CreateDepthCamera("camera_depth");
-  scene->DestroySensorByName("camera_depth");
-  depthCameraSensor = scene->CreateDepthCamera("camera_depth");
-  EXPECT_FALSE(scene->HasSensor(ConstSensorPtr()));
-  EXPECT_FALSE(scene->HasSensorId(8));
-  EXPECT_FALSE(scene->HasSensorName("invalid"));
-  EXPECT_TRUE(scene->HasSensorName("camera_depth"));
-
-  auto dCSensor = scene->SensorByName("camera_depth");
-  EXPECT_TRUE(scene->HasSensor(dCSensor));
-  EXPECT_TRUE(scene->HasSensorName("camera_depth"));
-  scene->DestroySensor(dCSensor, false);
-
-  depthCameraSensor = scene->CreateDepthCamera(76);
-  dCSensor = scene->SensorById(76);
-  EXPECT_TRUE(scene->HasSensor(dCSensor));
-  EXPECT_TRUE(scene->HasSensorId(76));
-  scene->DestroySensor(dCSensor, true);
-
-  depthCameraSensor = scene->CreateDepthCamera(76);
-  scene->DestroySensorById(76);
-
-  depthCameraSensor = scene->CreateDepthCamera();
-  scene->DestroySensorByIndex(0);
-
-  // lights
-  auto spotLight = scene->CreateSpotLight();
-  auto pointLight = scene->CreatePointLight("point_light");
-  auto directionalLight = scene->CreateDirectionalLight(99);
-
-  auto directionalLight2 = scene->LightById(99);
-  EXPECT_EQ(directionalLight, directionalLight2);
-
-  auto directionalLight3 = scene->CreateDirectionalLight("directional_light");
-
-  auto pointLight2 = scene->LightByName("point_light");
-  EXPECT_EQ(pointLight, pointLight2);
-
-  scene->DestroyLightByIndex(0);
-  scene->DestroyLight(pointLight, true);
-  scene->DestroyLights();
-
-  spotLight = scene->CreateSpotLight();
-  scene->DestroyNodeByIndex(0);
-
-  spotLight = scene->CreateSpotLight("light_node");
-  scene->DestroyNodeByName("light_node");
-
-  spotLight = scene->CreateSpotLight(56);
-  scene->DestroyNodeById(56);
-
   EXPECT_EQ(0u, scene->VisualCount());
-  EXPECT_EQ(0u, scene->SensorCount());
-  EXPECT_EQ(0u, scene->LightCount());
-
-  EXPECT_DOUBLE_EQ(0.0, scene->SimTime().Double());
-  scene->SetSimTime(common::Time(3.55));
-  EXPECT_DOUBLE_EQ(3.55, scene->SimTime().Double());
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -589,7 +526,7 @@ void SceneTest::NodeCycle(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -719,7 +656,120 @@ void SceneTest::Materials(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
+}
+
+/////////////////////////////////////////////////
+void SceneTest::Time(const std::string &_renderEngine)
+{
+  auto engine = rendering::engine(_renderEngine);
+  if (!engine)
+  {
+    igndbg << "Engine '" << _renderEngine << "' is not supported" << std::endl;
+    return;
+  }
+
+  auto scene = engine->CreateScene("scene");
+  ASSERT_NE(nullptr, scene);
+
+  std::chrono::steady_clock::duration duration =
+    std::chrono::steady_clock::duration::zero();
+
+  EXPECT_EQ(duration, scene->Time());
+
+  duration = std::chrono::seconds(23);
+
+  scene->SetTime(duration);
+  EXPECT_EQ(duration, scene->Time());
+
+  duration = std::chrono::seconds(1) + std::chrono::milliseconds(123);
+  scene->SetTime(duration);
+  EXPECT_EQ(duration, scene->Time());
+
+  duration = std::chrono::hours(24) +
+             std::chrono::seconds(6) +
+             std::chrono::milliseconds(123);
+  scene->SetTime(duration);
+  EXPECT_EQ(duration, scene->Time());
+
+  // Clean up
+  engine->DestroyScene(scene);
+  rendering::unloadEngine(engine->Name());
+}
+
+/////////////////////////////////////////////////
+void SceneTest::BackgroundMaterial(const std::string &_renderEngine)
+{
+  auto engine = rendering::engine(_renderEngine);
+  if (!engine)
+  {
+    igndbg << "Engine '" << _renderEngine << "' is not supported" << std::endl;
+    return;
+  }
+
+  auto scene = engine->CreateScene("scene");
+  ASSERT_NE(nullptr, scene);
+
+  EXPECT_EQ(nullptr, scene->BackgroundMaterial());
+
+  rendering::MaterialPtr mat = scene->CreateMaterial("test_mat");
+  scene->SetBackgroundMaterial(mat);
+  EXPECT_EQ(mat, scene->BackgroundMaterial());
+
+  scene->SetBackgroundMaterial(nullptr);
+  EXPECT_EQ(nullptr, scene->BackgroundMaterial());
+
+  // Clean up
+  engine->DestroyScene(scene);
+  rendering::unloadEngine(engine->Name());
+}
+
+/////////////////////////////////////////////////
+void SceneTest::Sky(const std::string &_renderEngine)
+{
+  auto engine = rendering::engine(_renderEngine);
+  if (!engine)
+  {
+    igndbg << "Engine '" << _renderEngine << "' is not supported" << std::endl;
+    return;
+  }
+
+  if (_renderEngine != "ogre2")
+  {
+    igndbg << "Sky not supported yet in rendering engine: "
+            << _renderEngine << std::endl;
+    return;
+  }
+
+  auto scene = engine->CreateScene("scene");
+  ASSERT_NE(nullptr, scene);
+
+  EXPECT_FALSE(scene->SkyEnabled());
+
+  scene->SetSkyEnabled(false);
+  EXPECT_FALSE(scene->SkyEnabled());
+
+  scene->SetSkyEnabled(true);
+  EXPECT_TRUE(scene->SkyEnabled());
+
+  scene->SetSkyEnabled(false);
+  EXPECT_FALSE(scene->SkyEnabled());
+
+  // set background material and verify sky remains disabled
+  rendering::MaterialPtr mat = scene->CreateMaterial("test_mat");
+  scene->SetBackgroundMaterial(mat);
+  EXPECT_EQ(mat, scene->BackgroundMaterial());
+  EXPECT_FALSE(scene->SkyEnabled());
+
+  // enable sky and verify it is not affected by background material
+  scene->SetSkyEnabled(true);
+  EXPECT_TRUE(scene->SkyEnabled());
+  scene->SetBackgroundMaterial(nullptr);
+  EXPECT_TRUE(scene->SkyEnabled());
+
+  // Clean up
+  engine->DestroyScene(scene);
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -758,9 +808,27 @@ TEST_P(SceneTest, Materials)
   Materials(GetParam());
 }
 
+/////////////////////////////////////////////////
+TEST_P(SceneTest, Time)
+{
+  Time(GetParam());
+}
+
+/////////////////////////////////////////////////
+TEST_P(SceneTest, BackgroundMaterial)
+{
+  BackgroundMaterial(GetParam());
+}
+
+/////////////////////////////////////////////////
+TEST_P(SceneTest, Sky)
+{
+  Sky(GetParam());
+}
+
 INSTANTIATE_TEST_CASE_P(Scene, SceneTest,
     RENDER_ENGINE_VALUES,
-    PrintToStringParam());
+    ignition::rendering::PrintToStringParam());
 
 int main(int argc, char **argv)
 {
