@@ -20,13 +20,13 @@
 #include <X11/Xresource.h>
 #endif
 
-#include "gz/math/Plane.hh"
-#include "gz/math/Vector2.hh"
-#include "gz/math/Vector3.hh"
+#include "ignition/math/Plane.hh"
+#include "ignition/math/Vector2.hh"
+#include "ignition/math/Vector3.hh"
 
-#include "gz/rendering/Camera.hh"
-#include "gz/rendering/RayQuery.hh"
-#include "gz/rendering/Utils.hh"
+#include "ignition/rendering/Camera.hh"
+#include "ignition/rendering/RayQuery.hh"
+#include "ignition/rendering/Utils.hh"
 
 namespace ignition
 {
@@ -91,7 +91,7 @@ math::Vector3d screenToPlane(
   _rayQuery->SetFromCamera(
       _camera, math::Vector2d(nx, ny));
 
-  gz::math::Planed plane(gz::math::Vector3d(0, 0, 1), offset);
+  ignition::math::Planed plane(ignition::math::Vector3d(0, 0, 1), offset);
 
   math::Vector3d origin = _rayQuery->Origin();
   math::Vector3d direction = _rayQuery->Direction();
@@ -104,11 +104,16 @@ float screenScalingFactor()
 {
   // todo(anyone) set device pixel ratio for high dpi displays on Windows
   float ratio = 1.0;
-  // the scaling factor seems to cause issues with mouse picking.
-  // see https://github.com/ignitionrobotics/ign-gazebo/issues/147
-#if 0
-  Display *disp = XOpenDisplay(nullptr);
-  char *resourceString = XResourceManagerString(disp);
+#ifdef __linux__
+  auto closeDisplay = [](Display * display)
+  {
+    if (display)
+      XCloseDisplay(display);
+  };
+  auto display =
+    std::unique_ptr<Display, decltype(closeDisplay)>(
+      XOpenDisplay(nullptr), closeDisplay);
+  char *resourceString = XResourceManagerString(display.get());
 
   if (resourceString)
   {
@@ -141,8 +146,8 @@ float screenScalingFactor()
     // We can use either the width or height in the following line. The zero
     // values in DisplayHeight and DisplayHeightMM is the screen number. A
     // value of zero uses the default screen.
-    float yDpiRes = (DisplayHeight(disp, 0) * 25.4) /
-      DisplayHeightMM(disp, 0);
+    float yDpiRes = (DisplayHeight(display.get(), 0) * 25.4) /
+      DisplayHeightMM(display.get(), 0);
 
     if (!math::equal(dpiDesktop, 0.0f) && !math::equal(yDpiRes, 0.0f))
       ratio = dpiDesktop / yDpiRes;
@@ -154,6 +159,73 @@ float screenScalingFactor()
 #endif
   return ratio;
 }
+
+/////////////////////////////////////////////////
+ignition::math::AxisAlignedBox transformAxisAlignedBox(
+    const ignition::math::AxisAlignedBox &_bbox,
+    const ignition::math::Pose3d &_pose)
+{
+  auto center = _bbox.Center();
+
+  // Get the 8 corners of the bounding box.
+  std::vector<ignition::math::Vector3d> vertices;
+  vertices.push_back(center + ignition::math::Vector3d(-_bbox.XLength()/2.0,
+                                                       _bbox.YLength()/2.0,
+                                                       _bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(_bbox.XLength()/2.0,
+                                                       _bbox.YLength()/2.0,
+                                                       _bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(-_bbox.XLength()/2.0,
+                                                       -_bbox.YLength()/2.0,
+                                                       _bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(_bbox.XLength()/2.0,
+                                                       -_bbox.YLength()/2.0,
+                                                       _bbox.ZLength()/2.0));
+
+  vertices.push_back(center + ignition::math::Vector3d(-_bbox.XLength()/2.0,
+                                                       _bbox.YLength()/2.0,
+                                                       -_bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(_bbox.XLength()/2.0,
+                                                       _bbox.YLength()/2.0,
+                                                       -_bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(-_bbox.XLength()/2.0,
+                                                       -_bbox.YLength()/2.0,
+                                                       -_bbox.ZLength()/2.0));
+  vertices.push_back(center + ignition::math::Vector3d(_bbox.XLength()/2.0,
+                                                       -_bbox.YLength()/2.0,
+                                                       -_bbox.ZLength()/2.0));
+
+
+  // Transform corners.
+  for (unsigned int i = 0; i < vertices.size(); ++i)
+  {
+    auto &v = vertices[i];
+    v = _pose.Rot() * v + _pose.Pos();
+  }
+
+  ignition::math::Vector3d min = vertices[0];
+  ignition::math::Vector3d max = vertices[0];
+
+  // find min / max of vertices
+  for (unsigned int i = 1; i < vertices.size(); ++i)
+  {
+    auto &v = vertices[i];
+
+    if (min.X() > v.X())
+      min.X() = v.X();
+    if (max.X() < v.X())
+      max.X() = v.X();
+    if (min.Y() > v.Y())
+      min.Y() = v.Y();
+    if (max.Y() < v.Y())
+      max.Y() = v.Y();
+    if (min.Z() > v.Z())
+      min.Z() = v.Z();
+    if (max.Z() < v.Z())
+      max.Z() = v.Z();
+  }
+  return ignition::math::AxisAlignedBox(min, max);
+}
 }
 }
 }
