@@ -17,21 +17,27 @@
 
 #include <gtest/gtest.h>
 
-#include <gz/common/Console.hh>
+#include <ignition/common/Console.hh>
 
 #include "test_config.h"  // NOLINT(build/include)
 
-#include "gz/rendering/Camera.hh"
-#include "gz/rendering/RenderEngine.hh"
-#include "gz/rendering/RenderingIface.hh"
-#include "gz/rendering/Scene.hh"
+#include "ignition/rendering/Camera.hh"
+#include "ignition/rendering/RenderEngine.hh"
+#include "ignition/rendering/RenderingIface.hh"
+#include "ignition/rendering/Scene.hh"
 
-using namespace gz;
+using namespace ignition;
 using namespace rendering;
 
 class SceneTest: public testing::Test,
                  public testing::WithParamInterface<const char *>
 {
+  // Documentation inherited
+  public: void SetUp() override
+  {
+    ignition::common::Console::SetVerbosity(4);
+  }
+
   // Test adding and removing visuals
   // Simulates 'levels' where visuals are added and removed throughout
   // frame updates
@@ -136,7 +142,7 @@ void SceneTest::AddRemoveVisuals(const std::string &_renderEngine)
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -148,6 +154,14 @@ void SceneTest::VisualAt(const std::string &_renderEngine)
             << _renderEngine << std::endl;
     return;
   }
+  else if (_renderEngine == "ogre2")
+  {
+    // VisualAt tests fail on CI, see issue #170
+    // https://github.com/ignitionrobotics/ign-rendering/issues/170
+    igndbg << "VisualAt test is disabled in " << _renderEngine << "."
+           << std::endl;
+    return;
+  }
 
   // create and populate scene
   RenderEngine *engine = rendering::engine(_renderEngine);
@@ -199,25 +213,25 @@ void SceneTest::VisualAt(const std::string &_renderEngine)
   camera->Update();
 
   // test get sphere object
-  math::Vector2i spherePosition(220, 307);
+  ignition::math::Vector2i spherePosition(220, 307);
   VisualPtr sphere_visual = scene->VisualAt(camera, spherePosition);
   ASSERT_TRUE(sphere_visual != nullptr);
   EXPECT_EQ("sphere", sphere_visual->Name());
 
   // test get box object
-  math::Vector2i boxPosition(452, 338);
+  ignition::math::Vector2i boxPosition(452, 338);
   VisualPtr box_visual = scene->VisualAt(camera, boxPosition);
   ASSERT_TRUE(box_visual != nullptr);
   EXPECT_EQ("box", box_visual->Name());
 
   // test get no object
-  math::Vector2i emptyPosition(300, 150);
+  ignition::math::Vector2i emptyPosition(300, 150);
   VisualPtr empty_visual = scene->VisualAt(camera, emptyPosition);
   ASSERT_TRUE(empty_visual == nullptr);
 
   // Clean up
   engine->DestroyScene(scene);
-  unloadEngine(engine->Name());
+  rendering::unloadEngine(engine->Name());
 }
 
 /////////////////////////////////////////////////
@@ -235,7 +249,7 @@ TEST_P(SceneTest, VisualAt)
 // It doesn't suppot optix just yet
 INSTANTIATE_TEST_CASE_P(Scene, SceneTest,
     RENDER_ENGINE_VALUES,
-    PrintToStringParam());
+    ignition::rendering::PrintToStringParam());
 
 int main(int argc, char **argv)
 {
